@@ -105,16 +105,21 @@ submix_t* submix_new (thread_info_t *info, int id, char* name){
 	free(tmp);
 
 	submix->id = id;
-	submix->name = (char*) malloc(sizeof(char) * strlen(name));
+/*	printf ("registering id:%d\n",id);*/
+	submix->name = (char*) malloc(sizeof(char) * (strlen(name)+1));
 	strcpy (submix->name,name);
 
 	return submix;
 }
 
 void submix_append(thread_info_t* info,submix_t* mix, int id){
+	int i;
+	int s = info->nmixes;
+
 	if ((id  + 1) > info->nmixes){	
 		info->nmixes = id + 1;
 		info->submixes = (submix_t**) realloc(info->submixes, sizeof(submix_t*) * info->nmixes);
+		for (i = s; i < info->nmixes; i++) info->submixes[i] = NULL;
 	}
 
 	*(info->submixes + id) = mix;
@@ -266,7 +271,8 @@ int process (jack_nframes_t nframes, void *arg){
                
 	info->inside = -1; 
         info->last_nframes = nframes;
-        
+       
+
         if (info->submixes){ 
                 for (i = 0; i < info->nmixes;i++){
                         if (info->submixes[i]){
@@ -352,6 +358,7 @@ static void* mix_thread(void *arg){
         jack_set_error_function ((void*)errfunc);
 
         while (!info->jack_info->client && (cc < 99)){
+		memset(&clientname,0,64);
                 sprintf(clientname,"kluppe%d",++cc);
                 info->jack_info->client = jack_client_new (clientname);
                 info->jack_info->client = info->jack_info->client;
@@ -366,7 +373,7 @@ static void* mix_thread(void *arg){
 
         printf ("jack client name: %s\n",clientname);
 
-        info->jack_info->name = malloc(sizeof(char) * (strlen(clientname) + 1));
+        info->jack_info->name = calloc(sizeof(char) , (strlen(clientname) + 1));
         strncpy (info->jack_info->name,clientname,strlen(clientname)) ;
 
 
@@ -443,9 +450,6 @@ static void* mix_thread(void *arg){
 					name = (char*) malloc (MAX_NAMESIZE * sizeof(char));
 					sprintf (name,"act%d",data->id);
 					submix = submix_new (info,data->id,name);
-					submix_append (info,submix,data->id);
-					free(name);
-					info->outside = 307;
 					/* connecting takes a while and can cause xruns */
 					/* LATER: move connecting to gui thread / looper.c */
 					if ((ports = jack_get_ports (info->jack_info->client, NULL, NULL, 
@@ -460,9 +464,12 @@ static void* mix_thread(void *arg){
 					}else{
 						printf ("can't find physical input ports for looper\n");
 					}
+					submix_append (info,submix,data->id);
+                                        free(name);
+                                        info->outside = 307;
 				}
 				info->outside = 309;
-	
+				
 				if (data->buf && data->id){
 					if (data->buf->status == BUFFER_STATUS_READY){
 						info->outside = 310;
@@ -534,7 +541,7 @@ static void* mix_thread(void *arg){
 									if (data->buf->normalized){
 										looperdata_lock(data2);
 										looperdata_write_dirtyblock(
-											data2,0.,(double)(data2->buf->size));
+											data2,0.,(double)(buffer_get_size(data2->buf)));
 										looperdata_unlock(data2);
 									}else{
 										newrecpos = looperdata_get_recpos(data);
