@@ -14,12 +14,22 @@ typedef struct _dirty_block {
 	double end;
 } dirty_block_t;
 
+/* predefine:*/
+static void looperdata_reset_grain (looper_data_t* data, int grain);
+static void inform_buffer(looper_data_t* data);
+
 looper_data_t* looperdata_new(buffer_info_t* buf, long samplerate,int id){
 	int i;
 	looper_data_t *data = (looper_data_t*) malloc (sizeof(looper_data_t));
 
 	data->buf = buf;
+	data->reverb = buffer_new(id,samplerate);
+	buffer_set_channels(data->reverb,2);
+	data->reverblength = 0;
+	data->feedback = 0.;
+	data->revpos = 0;
 	data->id = id;
+	data->buffer_group = 0;
 	data->samplerate = samplerate;
 	data->speed = 1.;
 	data->modespeed = 1.;
@@ -45,8 +55,9 @@ looper_data_t* looperdata_new(buffer_info_t* buf, long samplerate,int id){
 	data->recstart = 0;
 	data->recend = 0;
 	if (data->buf){
-		data->loopend = data->buf->size;
-		data->recend = data->buf->size;
+		data->buffer_group = buffer_get_group(data->buf,DS_PRIORITY_NORMAL);
+		data->loopend = buffer_get_size(data->buf);
+		data->recend = buffer_get_size(data->buf);
 	}
 	data->recpos = 0.;
 	data->isplaying = 0;
@@ -62,6 +73,7 @@ looper_data_t* looperdata_new(buffer_info_t* buf, long samplerate,int id){
 	data->recvuL = .0;
 	data->recvuR = .0;
 	data->playmode = LOOP_PLAYMODE_LOOP;
+	data->recmode = LOOP_RECMODE_LOOP;
 	data->cm_size = 1024;
 	data->cm_min = 0.001;
 	data->cm_max = 5.;
@@ -89,6 +101,7 @@ looper_data_t* looperdata_new(buffer_info_t* buf, long samplerate,int id){
 	data->grainspeeds = NULL;
 	data->grainsactive = NULL;
 	data->grainpitchtable = NULL;
+	data->grainpitchvolumes = NULL;
 
 	data->dirtylist = speciallist_new();
 	pthread_mutex_init (&data->looperdatamutex, NULL);
@@ -99,6 +112,7 @@ void looperdata_free(looper_data_t* data){
 	/* LATER free dirty block list */
 
 	speciallist_destroy(data->dirtylist);
+	buffer_delete(data->reverb);
 	free (data->customplaymode);
 	free (data->grainindexes);
 	free (data->grainoffsets);
@@ -107,6 +121,7 @@ void looperdata_free(looper_data_t* data){
 	free (data->grainspeeds);
 	free (data->grainsactive);
 	free (data->grainpitchtable);
+	free (data->grainpitchvolumes);
 
 	free (data);
 	data = NULL;
@@ -156,35 +171,36 @@ void looperdata_copy_settings (looper_data_t* dst, looper_data_t* src){
 			dst->isrecording = src->isrecording;
 			dst->playpos = src->playpos;
 			dst->playindex = src->playindex;
-			if (dst->playpos > dst->buf->size){
-				dst->playpos = dst->buf->size;
+			if (dst->playpos > buffer_get_size(dst->buf)){
+				dst->playpos = buffer_get_size(dst->buf);
 			}
-			if (dst->playindex > dst->buf->size){
-                                dst->playindex = dst->buf->size;
+			if (dst->playindex > buffer_get_size(dst->buf)){
+                                dst->playindex = buffer_get_size(dst->buf);
                         }
 			dst->recpos = src->recpos;
 			dst->pointer = src->pointer;
-			if (dst->recpos > dst->buf->size){
-                                dst->recpos = dst->buf->size;
+			if (dst->recpos > buffer_get_size(dst->buf)){
+                                dst->recpos = buffer_get_size(dst->buf);
 			}
 			dst->loopstart = src->loopstart;
-			if (dst->loopstart > dst->buf->size){
+			if (dst->loopstart > buffer_get_size(dst->buf)){
                                 dst->loopstart = 0; 
 				/* LATER find a mor intelligent solution */
                         }
 			dst->loopend = src->loopend;	
-			if (dst->loopend > dst->buf->size){
-				dst->loopend = dst->buf->size;
+			if (dst->loopend > buffer_get_size(dst->buf)){
+				dst->loopend = buffer_get_size(dst->buf);
 			}
 			dst->recstart = src->recstart;
-			if (dst->recstart > dst->buf->size){
+			if (dst->recstart > buffer_get_size(dst->buf)){
                                 dst->recstart = 0;
 				/* LATER find a mor intelligent solution */
                         }
 			dst->recend = src->recend;
-			if (dst->recend > dst->buf->size){
-                                dst->recend = dst->buf->size;
+			if (dst->recend > buffer_get_size(dst->buf)){
+                                dst->recend = buffer_get_size(dst->buf);
                         }
+        		inform_buffer(dst);
 		}	
 
 		if (dst->cm_size != src->cm_size){
@@ -200,7 +216,17 @@ void looperdata_copy_settings (looper_data_t* dst, looper_data_t* src){
 		dst->mingrainspeed = src->mingrainspeed;
 		dst->maxgrainspeed = src->maxgrainspeed;
 		looperdata_set_ngrains(dst,src->ngrains);
-		/* LATER: should probably copy grainpitchtable */
+
+/*		printf ("size:%ld\n",buffer_get_size(src->reverb));*/
+		buffer_resize(dst->reverb,buffer_get_size(src->reverb));
+		dst->reverblength = src->reverblength;
+		dst->feedback = src->feedback;
+
+		for (i = 0; i < src->ngrains; i++){
+			looperdata_reset_grain(dst,i);	
+			dst->grainpitchtable[i] = src->grainpitchtable[i];
+			dst->grainpitchvolumes[i] = src->grainpitchvolumes[i];
+		}
 	}
 }
 
@@ -287,6 +313,8 @@ int looperdata_read_once_dirtyblock(looper_data_t* data, double* start, double*
 		*start = block->start;
 		*end = block->end;
 		speciallist_delete(data->dirtylist, (void*)block);
+		/* LATER: is this right? */
+		free(block); 
 		return 1;
 	}		
 
@@ -295,6 +323,23 @@ int looperdata_read_once_dirtyblock(looper_data_t* data, double* start, double*
 	return 0;
 }
 
+void looperdata_set_playing(looper_data_t* data, int isplaying){
+	if (data) data->isplaying = isplaying;
+}
+void looperdata_set_recording(looper_data_t* data, int isrecording){
+	if (data) data->isrecording = isrecording;
+}
+
+int looperdata_get_playing(looper_data_t* data){
+	if (data) return data->isplaying;
+	else return 0;
+}
+int looperdata_get_recording(looper_data_t* data){
+	if (data) return data->isrecording;
+        else return 0;
+}
+
+
 void looperdata_calc_pan (looper_data_t* data){
 	if (data->panswing > 0.){
 		data->panpos += data->pandir 
@@ -373,6 +418,10 @@ void looperdata_set_recmix(looper_data_t* data, float recmix){
 	}
 }	
 
+float looperdata_get_recmix(looper_data_t* data){
+	return data->recmix;
+}
+
 int looperdata_get_midichannel (looper_data_t* data){
 	/* muse and my keyboard send 0 when they write 1 ?!? */
 	return (data->midichannel - 1);
@@ -388,12 +437,14 @@ void looperdata_set_mingrainspeed(looper_data_t* data,double speed){
 	if (data){
 		data->mingrainspeed = speed;
 	}
+        inform_buffer(data);
 }
 
 void looperdata_set_maxgrainspeed(looper_data_t* data,double speed){
         if (data){
                 data->maxgrainspeed = speed;
         }
+        inform_buffer(data);
 }
 
 void looperdata_set_mingrainlength(looper_data_t* data,long length){
@@ -403,6 +454,7 @@ void looperdata_set_mingrainlength(looper_data_t* data,long length){
 		if (data->maxgrainlength < data->mingrainlength)
                         data->maxgrainlength = data->mingrainlength;
         }
+        inform_buffer(data);
 }
 
 void looperdata_set_maxgrainlength(looper_data_t* data,long length){
@@ -411,6 +463,7 @@ void looperdata_set_maxgrainlength(looper_data_t* data,long length){
 		if (data->maxgrainlength < data->mingrainlength)
 			data->mingrainlength = data->maxgrainlength;
         }
+        inform_buffer(data);
 }
 
 void looperdata_set_graindensity(looper_data_t* data,int density){
@@ -547,6 +600,7 @@ static void looperdata_reset_grain (looper_data_t* data, int grain){
         }
 	if (a < 1) a = 1;
         data->grainmixfactor =  1.0f / sqrt((float)a);
+/*	printf ("mixfactor:%lf\n", data->grainmixfactor);	*/
 }
 
 void looperdata_set_grainpitchmode ( looper_data_t* data, int mode){
@@ -556,19 +610,25 @@ void looperdata_set_grainpitchmode ( looper_data_t* data, int mode){
 		data->grainpitchmode = GRAINPITCH_AUTO;
 		for (i = 0; i <  data->ngrains; i++){
 			*(data->grainpitchtable + i) = 1.;
+			*(data->grainpitchvolumes + i) = 1.;
 		}
+		data->grainpitchbend = 1.;
 	}else if (mode == GRAINPITCH_CHORD){
 		data->grainpitchmode = GRAINPITCH_CHORD;
 		 for (i = 0; i <  data->ngrains; i++){
                         *(data->grainpitchtable + i) = 0.;
+			*(data->grainpitchvolumes + i) = 0.;
                 }
+		data->grainpitchbend = 1.;
 	}else{
 		data->grainpitchmode = GRAINPITCH_RANGE;
 		for (i = 0; i <  data->ngrains; i++){
                         *(data->grainpitchtable + i) = 1.;
-			data->grainpitchbend = 1.;
+			*(data->grainpitchvolumes + i) = 1.;
                 }
+		data->grainpitchbend = 1.;
 	}
+        inform_buffer(data);
 }
 
 int looperdata_get_grainpitchmode (looper_data_t* data){
@@ -579,32 +639,70 @@ void looperdata_set_grainpitchtablevalue ( looper_data_t* data, double value, do
 	int i = 0;
 	int found = 0;
 
-/*	printf ("set grainpitch\n");	*/
+/*	printf ("set grainpitch:%lf - %lf\n",value,vol);	*/
 	if (vol == 0.){
 		while (!found && (i < data->ngrains)){
 			if (*(data->grainpitchtable + i) == value){
-				if (data->grainpitchmode == GRAINPITCH_CHORD)
-					*(data->grainpitchtable + i) = 0.; /* fall back to regular */
-				else 	*(data->grainpitchtable + i) = 1.; /* mute */
+				if (data->grainpitchmode == GRAINPITCH_CHORD){
+					*(data->grainpitchtable + i) = 0.; /* mute */
+					*(data->grainpitchvolumes + i) = 0.;
+				}else{
+				 	*(data->grainpitchtable + i) = 1.; /* fall back to regular volume */
+					*(data->grainpitchvolumes + i) = 0;
+				}
 				found++;
 			}
 			i++;
 		}		
 	}else{
 		while (!found && (i < data->ngrains)){
-			if ((*(data->grainpitchtable + i) == 1.) || (*(data->grainpitchtable + i) == 0.)){
-				*(data->grainpitchtable + i) = value;
-				found++;
+			if (data->grainpitchmode == GRAINPITCH_CHORD){
+				if (*(data->grainpitchtable + i) == 0.){
+					*(data->grainpitchtable + i) = value;
+					*(data->grainpitchvolumes + i) = vol;
+                                	found++;
+				}
+			}else{
+				if (*(data->grainpitchtable + i) == 1.){
+                                        *(data->grainpitchtable + i) = value;
+					*(data->grainpitchvolumes + i) = vol;
+                                        found++;
+                                }
 			}
 			i++;
 		}
 	}
 }
 
+static void inform_buffer(looper_data_t* data){
+	long bw = 0;
+	if (!data->buf) return;
+	if (buffer_get_type(data->buf) != BUFFER_TYPE_DISCSTREAM) return;
+
+	if (data->ngrains > 1){
+		/* calculate the maximum distance between two grain-playheads:*/
+		long slow, fast, cursor;
+		long tmp, offset;
+		offset = data->maxgrainlength * (data->ngrains + 1);
+		offset /= 15;
+		slow = (long)((float)data->maxgrainlength * data->mingrainspeed);
+		fast = (long)((float)data->maxgrainlength * data->maxgrainspeed);
+		cursor = (long)((float)data->maxgrainlength * data->speed);
+		bw = abs(cursor - slow) + offset;
+		tmp = abs(fast - slow) + offset;
+		if (tmp > bw) bw = tmp;
+/*		printf ("max distance between two grains:%ld\n",bw);*/
+	}
+
+	buffer_set_area(data->buf,data->buffer_group,
+			data->loopstart,data->loopend,bw);
+}
+
 void looperdata_set_grainpitchbend (looper_data_t* data, double bend){
 /*	printf ("set grainpitchbend\n");*/
 	if ((bend > -1.) && (bend < 2.))
 		data->grainpitchbend = bend;
+	inform_buffer(data);
 }
 
 void looperdata_set_loopstart(looper_data_t* data, long start){
@@ -613,6 +711,7 @@ void looperdata_set_loopstart(looper_data_t* data, long start){
 		if (data->loopstart >= data->loopend)
 			data->loopend  = data->loopstart + 1;
 	}
+        inform_buffer(data);
 }
 
 void looperdata_set_loopend(looper_data_t* data, long end){
@@ -621,6 +720,7 @@ void looperdata_set_loopend(looper_data_t* data, long end){
 		if (data->loopstart >= data->loopend)
 			data->loopend  = data->loopstart + 1;
 	}
+        inform_buffer(data);
 }
 
 void looperdata_set_playmode(looper_data_t* data,int mode){
@@ -630,6 +730,10 @@ void looperdata_set_playmode(looper_data_t* data,int mode){
 	else 	data->modespeed = 1.;
 }
 
+void looperdata_set_recmode(looper_data_t* data,int mode){
+        data->recmode = mode;
+}
+
 long looperdata_get_loopstart(looper_data_t* data){
         return data->loopstart;
 }    
@@ -688,7 +792,7 @@ int looperdata_get_limiter(looper_data_t* data){
 	return data->limiter;
 }
 
-void looperdata_aet_limiter(looper_data_t* data, int limiter){
+void looperdata_set_limiter(looper_data_t* data, int limiter){
 	if (limiter) data->limiter = 1;
 	else data->limiter = 0;
 }
@@ -752,87 +856,83 @@ void looperdata_set_ngrains (looper_data_t* data, int ngrains){
 		data->grainspeeds  = realloc (data->grainspeeds, sizeof(double) * ngrains);
 		data->grainsactive = realloc (data->grainsactive, sizeof(int) * ngrains);
 		data->grainpitchtable = realloc (data->grainpitchtable, sizeof(double) * ngrains);
+		data->grainpitchvolumes = realloc (data->grainpitchvolumes, sizeof(double) * ngrains);
 
 		if (data->buf){
 			for (i = data->ngrains; i < ngrains; i++){
 				looperdata_reset_grain(data,i);
-				if (data->grainpitchmode == GRAINPITCH_CHORD)
+				if (data->grainpitchmode == GRAINPITCH_CHORD){
 					data->grainpitchtable[i] = 0.;
-				else 	data->grainpitchtable[i] = 1.;
+					data->grainpitchvolumes[i] = 0.;
+				}else{
+				 	data->grainpitchtable[i] = 1.;
+					data->grainpitchvolumes[i] = 1.;
+				}
 /*				if (data->grainsactive[i]) a++;*/
 			}
 		}
 /*		data->grainmixfactor =  1.0f / (1. + log10f(sqrt(ngrains)) * 3.333);*/
 		data->grainmixfactor =  1.0f / sqrt(ngrains);
+/*		printf ("mixfactor:%lf\n",data->grainmixfactor);*/
 		/* change = 20 * log10(sqrt(number)) db; 6db = factor 2 */
 	}
 	data->ngrains = ngrains;
+        inform_buffer(data);
 }
 
 void looperdata_write_sample(looper_data_t* data,float* L, float* R){
-	float *Ll;
-	float *Rl;
+	if (!data->buf) return;
+	if (!buffer_get_size(data->buf))  return;
 
-	if (data->buf->channels > 1){
-		if (data->inmaxL < fabs(*L)){data->inmaxL = fabs(*L);}
-		if (data->inmaxR < fabs(*R)){data->inmaxR = fabs(*R);}
-
-		buffer_lock(data->buf);
-		*(data->buf->L + (long)data->recpos) = *L * data->recmixnew + 
-			*(data->buf->L + (long)data->recpos) * data->recmixorig;
-		Ll = (data->buf->L + (long)data->recpos);
-
-		*(data->buf->R + (long)data->recpos) = *R * data->recmixnew + 
-			*(data->buf->R + (long)data->recpos) * data->recmixorig;
-		Rl = (data->buf->R + (long)data->recpos);
-/*
-                if (data->limiter){     
-                        if (*Ll > .5)            *Ll = 1. - .25 / *Ll;
-                        else if (*Ll < -.5)      *Ll = -1. - .25 / *Ll;
-                        if (*Rl > .5)            *Rl = 1. - .25 / *Rl;
-                        else if (*Rl < -.5)      *Rl = -1. - .25 / *Rl;
-                }
-*/
-                if (data->limiter){
-                        if (*L > data->limknee) *L = data->limit - data->limconst / *L;
-                        else if (*L < -data->limknee) *L = -data->limit - data->limconst / *L;
-                        if (*R > data->limknee) *R = data->limit - data->limconst / *R;
-                        else if (*R < -data->limknee) *R = -data->limit - data->limconst / *R;
-                }
-
-
-		buffer_unlock(data->buf);
-	}else{
-		if (data->inmaxL < fabs(*L)){data->inmaxL = fabs(*L);}
-		if (data->inmaxR < fabs(*R)){data->inmaxR = fabs(*R);}
-		buffer_lock(data->buf);
-		*(data->buf->L + (long)data->recpos) = (*L + *R) * S2M_FACTOR * data->recmixnew +
-                        *(data->buf->L + (long)data->recpos) * data->recmixorig;
-		Ll = (data->buf->L + (long)data->recpos);
-/*
-		if (data->limiter){
-			if (*Ll > .5)            *Ll = 1. - .25 / *Ll;
-                        else if (*Ll < -.5)      *Ll = -1. - .25 / *Ll;
+	if (data->isrecording){
+		if (buffer_get_channels(data->buf) > 1){
+			if (data->inmaxL < fabs(*L)){data->inmaxL = fabs(*L);}
+			if (data->inmaxR < fabs(*R)){data->inmaxR = fabs(*R);}
+	
+			buffer_lock(data->buf);	
+			buffer_writesample(data->buf, 1, (long)data->recpos, 0, 
+				buffer_readsample_interpolated(data->buf, 1, (long)data->recpos, data->buffer_group)
+				* data->recmixorig + *L * data->recmixnew);
+			buffer_writesample(data->buf, 2, (long)data->recpos, 0,  
+                        	buffer_readsample_interpolated(data->buf, 2, (long)data->recpos, data->buffer_group)
+                        	* data->recmixorig + *R * data->recmixnew);
+                	if (data->limiter){
+                        	if (*L > data->limknee) *L = data->limit - data->limconst / *L;
+                        	else if (*L < -data->limknee) *L = -data->limit - data->limconst / *L;
+                        	if (*R > data->limknee) *R = data->limit - data->limconst / *R;
+                        	else if (*R < -data->limknee) *R = -data->limit - data->limconst / *R;
+                	}
+	
+	
+			buffer_unlock(data->buf);
+		}else{
+			if (data->inmaxL < fabs(*L)){data->inmaxL = fabs(*L);}
+			if (data->inmaxR < fabs(*R)){data->inmaxR = fabs(*R);}
+	
+			buffer_lock(data->buf);
+			buffer_writesample(data->buf, 1, (long)data->recpos, 0,
+                        	buffer_readsample_interpolated(data->buf, 1, (long)data->recpos, data->buffer_group)
+                        	* data->recmixorig + (*L + *R) *  S2M_FACTOR * data->recmixnew);
+                	if (data->limiter){
+                        	if (*L > data->limknee) *L = data->limit - data->limconst / *L;
+                        	else if (*L < -data->limknee) *L = -data->limit - data->limconst / *L;
+                	}
+	
+			buffer_unlock(data->buf);
 		}
-*/
-                if (data->limiter){
-                        if (*L > data->limknee) *L = data->limit - data->limconst / *L;
-                        else if (*L < -data->limknee) *L = -data->limit - data->limconst / *L;
-                }
-
-		buffer_unlock(data->buf);
-	}
-
-	if (++data->recpos > data->recend){
-        	data->recpos = data->recstart;
-	}
-	if (data->recpos < data->recstart){
-        	data->recpos = data->recstart;
+		if (++data->recpos > data->recend){
+                	data->recpos = data->recstart;
+                	if (data->recmode == LOOP_RECMODE_ONCE)
+                        	data->isrecording = 0;
+        	}
+        	if (data->recpos < data->recstart){
+                	data->recpos = data->recstart;
+        	}
 	}
 }
 
 static double grain_inc(looper_data_t* data, int i){
-	double pos = 0.;
+	double pos = data->playpos;
 
         *(data->grainindexes + i) += 1;
 
@@ -853,18 +953,59 @@ static double grain_inc(looper_data_t* data, int i){
 	return pos;
 }
 
+/* reverb : */
+
+void looperdata_set_reverblength(looper_data_t* data, long length){
+	if (length < 0) length = 0;
+	if (length > 441000) length = 441000; /* any upper border for now */
+	if (data){
+		buffer_resize(data->reverb,length);
+		data->reverblength = buffer_get_size(data->reverb);
+	}
+}
+
+long looperdata_get_reverblength(looper_data_t* data){
+	if (data)
+		return data->reverblength;
+	else return 0;
+}
+
+void looperdata_set_feedback(looper_data_t* data, double feedback){
+	if (feedback < 0.) feedback = 0.;
+	if (feedback > 1.) feedback = 1.;
+/*	printf ("set feedback:%lf\n",feedback);*/
+	if (data)
+		data->feedback = feedback;
+}
+
+double looperdata_get_feedback(looper_data_t* data){
+	if (data)
+		return data->feedback;
+	else return 0;
+}
+
+
+/* process as sample ; LATER: do block functions */
+
 void looperdata_calc_sample(looper_data_t* data,float* L, float* R){
 	int i;
 	float vol = 1.;
 	double pos, ipos;
 
-	if (data->isplaying){
+	if (data->isplaying && data->buf){
 		if (data->ngrains < 1){
 			/* single playhead */
-			*L = inter4pol (data->buf->L, data->playpos, data->buf->size);
-			if (data->buf->channels > 1)
-       				*R = inter4pol (data->buf->R, data->playpos, data->buf->size);
+			/*
+			*L = inter4pol (data->buf->L, data->playpos, buffer_get_size(data->buf));
+			if (buffer_get_channels(data->buf) > 1)
+       				*R = inter4pol (data->buf->R, data->playpos, buffer_get_size(data->buf));
 			else 	*R = *L;
+			*/
+			pos = data->playpos;
+			*L = buffer_readsample_interpolated(data->buf,1,pos, data->buffer_group);
+			if (buffer_get_channels(data->buf) > 1)
+				*R = buffer_readsample_interpolated(data->buf,2,pos, data->buffer_group);
+			else    *R = *L;
 		}else{
 			/* granular playhead(s) */
 			*L = .0;
@@ -874,15 +1015,26 @@ void looperdata_calc_sample(looper_data_t* data,float* L, float* R){
 				if (*(data->grainsactive + i)){
 	                                if (*(data->grainindexes + i) < *(data->grainfadelengths + i))
                                         	vol = (float)*(data->grainindexes + i) / (float)*(data->grainfadelengths + i);
-                                	else if ( (ipos = (*(data->grainsizes + i) - *(data->grainindexes + i))) < *(data->grainfadelengths + i))
+                                	else if ( (ipos = (*(data->grainsizes + i) 
+						- *(data->grainindexes + i))) 
+						< *(data->grainfadelengths + i))
                                         	vol = ipos / (float)*(data->grainfadelengths + i);
                                 	else vol = 1.;
-                                	vol *= data->grainmixfactor;
+	
+                                	vol *= data->grainmixfactor * *(data->grainpitchvolumes + i);
 
-					*L += inter4pol (data->buf->L, pos, data->buf->size) * vol;
-					if (data->buf->channels > 1)
-						*R += inter4pol (data->buf->R, pos, data->buf->size) * vol;	
+/*
+					*L += inter4pol (data->buf->L, pos, buffer_get_size(data->buf)) * vol;
+					if (buffer_get_channels(data->buf) > 1)
+						*R += inter4pol (data->buf->R, pos, buffer_get_size(data->buf)) * vol;	
 					else 	*R = *L; 
+*/
+/*					if (pos == 0.) printf ("pos 0 : %d:%ld\n",i,*(data->grainoffsets + i));*/
+					*L += buffer_readsample_interpolated(data->buf,1,pos, data->buffer_group) * vol;
+					if (buffer_get_channels(data->buf) > 1)
+						*R += buffer_readsample_interpolated(data->buf,2,pos, data->buffer_group) * vol;
+					else    *R = *L; 
+
 				}
 			}	
 		}
@@ -891,7 +1043,7 @@ void looperdata_calc_sample(looper_data_t* data,float* L, float* R){
 				sin((data->playpos - data->loopstart) * PI / (data->loopend - data->loopstart)) * 1.5 + .5;
 		}else if (data->playmode == LOOP_PLAYMODE_CUSTOMSPEED){
 			data->modespeed = 
-				inter4pol(data->customplaymode,
+				inter4pol_float(data->customplaymode,
 				(float)(data->playpos - data->loopstart) * (float)data->cm_size / 
 				(float)(data->loopend - data->loopstart),data->cm_size);
 		}else if (data->playmode == LOOP_PLAYMODE_EXTERNALPOS){
@@ -917,10 +1069,14 @@ void looperdata_calc_sample(looper_data_t* data,float* L, float* R){
 				data->modespeed = 1;
 			}
                 	if ((data->speed * data->modespeed) > .0){
-                        	data->playindex += data->loopstart - data->loopend;
+				if (data->playmode == LOOP_PLAYMODE_ONCE){
+					data->playindex = data->loopstart;
+					looperdata_set_playing(data,0);
+                        	} else data->playindex += data->loopstart - data->loopend;
                 	}else{
                         	data->playindex = data->loopend;
                 	}
+/*			printf ("wrap around forward\n");*/
         	}else if (data->playindex < data->loopstart){
 			if (data->playmode == LOOP_PLAYMODE_BACKFORTH){
                         	data->modespeed *= -1;
@@ -931,12 +1087,15 @@ void looperdata_calc_sample(looper_data_t* data,float* L, float* R){
                 	if ((data->speed  * data->modespeed) > .0){
                         	data->playindex = data->loopstart;
                 	}else{
-                        	data->playindex += data->loopend - data->loopstart;
+				if (data->playmode == LOOP_PLAYMODE_ONCE){
+                                        data->playindex = data->loopend;
+                                        looperdata_set_playing(data,0);
+                                } else data->playindex += data->loopend - data->loopstart;
                 	}
         	}
 		if (data->playmode == LOOP_PLAYMODE_POSITION){
 			data->playpos = data->loopstart + (data->loopend - data->loopstart) / 
-				(data->cm_max - data->cm_min) * (inter4pol (data->customplaymode,
+				(data->cm_max - data->cm_min) * (inter4pol_float (data->customplaymode,
 				(data->playindex - data->loopstart) * data->cm_size /
 				(data->loopend - data->loopstart),data->cm_size) - data->cm_min);
  		}else{
@@ -945,19 +1104,37 @@ void looperdata_calc_sample(looper_data_t* data,float* L, float* R){
 
 		if (data->panswing)
 			looperdata_calc_pan(data);
-		*L *= data->Lfact;
-		*R *= data->Rfact;
 
+		/*final volume: */
+                *L *= data->Lfact;
+                *R *= data->Rfact;
+
+		/* reverb: */
+                if (data->reverblength > 0){
+                        if (++data->revpos >= data->reverblength) data->revpos = 0;
+			*L += buffer_readsample_interpolated(data->reverb, 1, data->revpos, 0);
+			*R += buffer_readsample_interpolated(data->reverb, 2, data->revpos, 0);
+			buffer_writesample(data->reverb, 1, data->revpos, 0, *L * data->feedback);
+			buffer_writesample(data->reverb, 2, data->revpos, 0, *R * data->feedback);
+/*
+			*L += *(data->reverb->L + data->revpos);
+			*R += *(data->reverb->R + data->revpos);
+                        *(data->reverb->L + data->revpos) = *L * data->feedback; 
+			*(data->reverb->R + data->revpos) = *R * data->feedback;
+*/
+                }
+
+		/* limiter: */
 		if (data->limiter){
 			if (*L > data->limknee)	*L = data->limit - data->limconst / *L;
 			else if (*L < -data->limknee) *L = -data->limit - data->limconst / *L;
 			if (*R > data->limknee) *R = data->limit - data->limconst / *R;
                         else if (*R < -data->limknee) *R = -data->limit - data->limconst / *R;
 		}
-	
+
+		/* prevent eternity */	
 		if (data->outmaxL < *L){data->outmaxL = *L;}
                 if (data->outmaxR < *R){data->outmaxR = *R;}
-		
 	}else{
 		*L = 0.;
 		*R = 0.;
