@@ -15,7 +15,8 @@ char* config_check_filename (char *filename){
 	return filename;
 }
 
-int save_config (GList *looperlist, speciallist_t *bufferlist, char* spath, char* cpath, char *filename){
+int save_config (GList *looperlist, speciallist_t *bufferlist, char* spath, 
+		char* cpath, char *filename, gint x, gint y, gint width, gint height){
 	xmlDocPtr doc;
 	int err;
 	char txt[128];
@@ -31,6 +32,14 @@ int save_config (GList *looperlist, speciallist_t *bufferlist, char* spath, char
 
 	xmlNewChild (topptr, NULL, (xmlChar*)"soundfilepath", (xmlChar*)spath);
 	xmlNewChild (topptr, NULL, (xmlChar*)"configfilepath", (xmlChar*)cpath);
+	sprintf (txt, "%d",(int)x);
+        xmlNewChild (topptr, NULL, (xmlChar*)"window-xpos",(xmlChar*)txt);
+	sprintf (txt, "%d",(int)y);
+        xmlNewChild (topptr, NULL, (xmlChar*)"window-ypos",(xmlChar*)txt);
+	sprintf (txt, "%d",(int)width);
+	xmlNewChild (topptr, NULL, (xmlChar*)"window-width",(xmlChar*)txt);
+	sprintf (txt, "%d",(int)height);
+        xmlNewChild (topptr, NULL, (xmlChar*)"window-height",(xmlChar*)txt);
 	
 	while (listptr){
 		if ((looper = (looper_gui_t*)listptr->data) != NULL){
@@ -107,8 +116,14 @@ int save_config (GList *looperlist, speciallist_t *bufferlist, char* spath, char
                                 sprintf (txt, "%.4lf",gtk_loopview_get_bpb(GTK_LOOPVIEW(looper->loopview)));
                                 xmlNewChild (looperptr, NULL, (xmlChar*)"bpb",(xmlChar*)txt);
 
+				sprintf (txt, "%ld",(long)gtk_loopview_get_gridoffset(GTK_LOOPVIEW(looper->loopview)));
+                                xmlNewChild (looperptr, NULL, (xmlChar*)"gridoffset",(xmlChar*)txt);
+
 				sprintf (txt, "%d", looper->data->playmode);
 				xmlNewChild (looperptr, NULL, (xmlChar*)"playmode",(xmlChar*)txt);
+				
+				sprintf (txt, "%d", looper->data->recmode);
+                                xmlNewChild (looperptr, NULL, (xmlChar*)"recmode",(xmlChar*)txt);
 
 				sprintf (txt, "%d", looper->data->ngrains);
 				xmlNewChild (looperptr, NULL, (xmlChar*)"ngrains",(xmlChar*)txt);
@@ -155,14 +170,15 @@ int save_config (GList *looperlist, speciallist_t *bufferlist, char* spath, char
 		xmlNewChild (bufferptr, NULL, (xmlChar*)"filename",(xmlChar*)buf->filename);
 		sprintf (txt, "%d", buf->status);
 		xmlNewChild (bufferptr, NULL, (xmlChar*)"status",(xmlChar*)txt);	
-		sprintf (txt, "%d", buf->channels);
+		sprintf (txt, "%d", buffer_get_type(buf));
+		xmlNewChild (bufferptr, NULL, (xmlChar*)"type",(xmlChar*)txt);
+                sprintf (txt, "%d", buffer_get_channels(buf));
 		xmlNewChild (bufferptr, NULL, (xmlChar*)"channels",(xmlChar*)txt);
-		sprintf (txt, "%ld", buf->size);
+		sprintf (txt, "%ld", buffer_get_size(buf));
 		xmlNewChild (bufferptr, NULL, (xmlChar*)"size",(xmlChar*)txt);
 		buf = speciallist_get_next(bufferlist,buf);
 	}
 
-
 	if ((err = xmlSaveFormatFile (filename, doc, TRUE)) == -1 ){
 		xmlFreeDoc (doc);
 		return -1;
@@ -268,6 +284,16 @@ static char* subnode2string (xmlNodePtr parent, const char *key){
         return NULL;
 }
 
+void config_get_geometry(configfile_handler_t* handler,gint* x, gint* y, gint* width,gint* height){
+/*        xmlNodePtr node;  */
+
+	*x= subnode2int (handler->kluppe,"window-xpos");
+	*y = subnode2int (handler->kluppe,"window-ypos");
+        *width = subnode2int (handler->kluppe,"window-width");
+        *height = subnode2int (handler->kluppe,"window-height");
+
+        return;
+}                       
 
 int config_next_looper (configfile_handler_t* handler){
         xmlNodePtr node;
@@ -355,6 +381,9 @@ int config_read_looper (configfile_handler_t* handler, looper_gui_t *looper){
 			looperdata_set_playmode(looper->data,
 				subnode2int(lptr,"playmode"));
 
+			looperdata_set_recmode(looper->data,
+                                subnode2int(lptr,"recmode"));
+
 /*			gtk_toggle_button_set_active((GtkToggleButton*)looper->limiterbutton,
 				(subnode2int(lptr,"limiter") ? TRUE : FALSE));
 */
@@ -383,6 +412,8 @@ int config_read_looper (configfile_handler_t* handler, looper_gui_t *looper){
 				subnode2double(lptr,"bpm"));
 			gtk_loopview_set_bpb(GTK_LOOPVIEW(looper->loopview),
                                 subnode2double(lptr,"bpb"));
+			gtk_loopview_set_gridoffset(GTK_LOOPVIEW(looper->loopview),
+				subnode2long(lptr,"gridoffset"));
 
 
 			switch (subnode2int(lptr,"playmode")){
@@ -406,6 +437,21 @@ int config_read_looper (configfile_handler_t* handler, looper_gui_t *looper){
                                         gtk_check_menu_item_set_active(
 						(GtkCheckMenuItem*)looper->customposmode, TRUE);
                                         break;
+				case LOOP_PLAYMODE_ONCE:
+                                        gtk_check_menu_item_set_active(
+                                                (GtkCheckMenuItem*)looper->oncemode, TRUE);
+                                        break;
+			}
+
+			switch (subnode2int(lptr,"recmode")){
+                                case LOOP_RECMODE_LOOP:
+                                        gtk_check_menu_item_set_active(
+                                                (GtkCheckMenuItem*)looper->recloopmode, TRUE);
+                                        break;
+				case LOOP_RECMODE_ONCE:
+                                        gtk_check_menu_item_set_active(
+                                                (GtkCheckMenuItem*)looper->reconcemode, TRUE);
+                                        break;
 			}
 			
 			if (subnode2int(lptr,"zoomend"))
@@ -440,25 +486,32 @@ int config_next_buffer (configfile_handler_t* handler){
 }
 
 int config_read_buffer (configfile_handler_t* handler, buffer_info_t* buf){
-	struct stat fstat;
+	int type = 0;
 	int file_ok = 0;
-	char *tmpchr;
+	char *filename;
+	struct stat fstat;
 
 	if (buf && handler->bufferptr){
-		tmpchr = subnode2string (handler->bufferptr,"filename");
-		if (tmpchr)
-			if (!stat(tmpchr,&fstat))
-				file_ok = 1;
-		if (file_ok){
-			buffer_loadfile(buf,tmpchr,handler->samplerate);
+		type = subnode2int (handler->bufferptr,"type");
+		filename = subnode2string (handler->bufferptr,"filename");
+
+		if (type == BUFFER_TYPE_DISCSTREAM){
+			buffer_open_discstream(buf,filename,handler->samplerate);
 		}else{
-			buffer_set_channels(buf, 
-				subnode2int(handler->bufferptr,"channels"));
-			buffer_resize(buf,
-				subnode2long(handler->bufferptr,"size"));
-			buffer_set_shortname(buf,
-                                subnode2string(handler->bufferptr,"shortname"));
-			buf->pending = BUFFER_STATUS_READY;
+			/* old configfiles don't have a type for buffers 
+							-> check for filenames */
+			if (filename && !stat(filename,&fstat)) file_ok = 1;
+		 	if ((type == BUFFER_TYPE_FILEBUFFER) || ((type == 0) && (file_ok == 1))){
+				buffer_loadfile(buf,filename,handler->samplerate);
+			}else{
+				buffer_set_channels(buf, 
+					subnode2int(handler->bufferptr,"channels"));
+				buffer_resize(buf,
+					subnode2long(handler->bufferptr,"size"));
+				buffer_set_shortname(buf,
+                               		subnode2string(handler->bufferptr,"shortname"));
+				buf->pending = BUFFER_STATUS_READY;
+			}
 		}
 	}
 	return 0;
