@@ -14,10 +14,11 @@ typedef struct _configfile_handler {
 	long samplerate;
 } configfile_handler_t;
 
-int save_config (GList *looperlist, speciallist_t *bufferlist, char* spath, char* cpath, char *filename);
+int save_config (GList *looperlist, speciallist_t *bufferlist, char* spath, char* cpath, char *filename, gint x, gint y, gint width, gint height);
 char* config_check_filename (char *filename);
 configfile_handler_t* open_config_reader (char *filename, long samplerate);
 void close_config_reader (configfile_handler_t* handler);
+void config_get_geometry(configfile_handler_t* handler,gint* x, gint* y, gint* width, gint* height);
 int config_next_looper (configfile_handler_t* handler);
 int config_read_looper (configfile_handler_t* handler, looper_gui_t *looper);
 int config_next_buffer (configfile_handler_t* handler);
