@@ -8,6 +8,7 @@
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <gtk/gtk.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
@@ -34,8 +35,8 @@ void gtk_loopview_set_update_policy (GtkLoopview      *loopview,GtkUpdateType  p
 void gtk_loopview_update_gplayposition (GtkLoopview *loopview);
 void gtk_loopview_set_looperdata (GtkLoopview      *loopview,looper_data_t *looperdata);
 void gtk_loopview_update_waveform (GtkLoopview *loopview);
-void gtk_loopview_calc_peaks(GtkLoopview *loopview,guint32 startx, guint32 endx);
-
+/*void gtk_loopview_calc_peaks(GtkLoopview *loopview,guint32 startx, guint32 endx);*/
+void gtk_loopview_copy_peaks(GtkLoopview *loopview, peakdata_t *pd);
 static void gtk_loopview_realize (GtkWidget *widget);
 static void gtk_loopview_size_request (GtkWidget      *widget, GtkRequisition *requisition);
 static void gtk_loopview_size_allocate (GtkWidget     *widget, GtkAllocation *allocation);
@@ -159,7 +160,9 @@ static void gtk_loopview_init (GtkLoopview *loopview) {
   	loopview->button = 0;
   	loopview->policy = GTK_UPDATE_CONTINUOUS;
   	loopview->timer = 0;
+	loopview->peakview = NULL;
   	loopview->looperdata = looperdata_new(NULL,44100, 0);
+/*	loopview->buffer_group = 0;*/
   	loopview->Lmin = NULL;
   	loopview->Rmin = NULL;
 	loopview->Lmax = NULL;
@@ -187,6 +190,7 @@ static void gtk_loopview_init (GtkLoopview *loopview) {
 	loopview->bpm = 120;
 	loopview->bpb = 4;
 	loopview->gridwidth = 22050;
+	loopview->gridoffset = 0;
 	loopview->ggridstart = 0;
 	loopview->ggridwidth = 1;
 	loopview->snap2grid = FALSE;
@@ -235,6 +239,7 @@ GtkWidget* gtk_loopview_new (looper_data_t * looperdata)
 
   	loopview = gtk_type_new (gtk_loopview_get_type ());
   	gtk_loopview_set_looperdata (loopview, looperdata);
+	gtk_loopview_update_waveform(loopview);
   	return GTK_WIDGET (loopview);
 }
 
@@ -276,11 +281,20 @@ void gtk_loopview_set_looperdata (GtkLoopview *loopview, looper_data_t *looperda
   	loopview->looperdata = looperdata;
 	loopview->zoomstart = 0;
 	loopview->zoomend = 0;
+/*	if (loopview->peakview) peakview_delete(loopview->peakview);*/
 	if (loopview->looperdata->buf){
-		loopview->zoomend = loopview->looperdata->buf->size;
+		loopview->zoomend = buffer_get_size(loopview->looperdata->buf);
+		if (!loopview->peakview)
+			loopview->peakview = peakview_new(loopview->looperdata->buf);
+		else peakview_set_buffer(loopview->peakview, loopview->looperdata->buf);
+		peakview_set_area(loopview->peakview,loopview->zoomstart, 
+						loopview->zoomend, loopview->npeaks);
+	}else{
+		gtk_loopview_update_area(loopview,0,loopview->npeaks);
 	}
 	gtk_loopview_set_bpm(loopview,120);
-	gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);
+/*	peakview_calculate(loopview->peakview,loopview->zoomstart, loopview->zoomend);*/
+/*	gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);*/
 
 /*	gtk_loopview_update (loopview);*/
 }
@@ -289,7 +303,10 @@ void gtk_loopview_update_waveform (GtkLoopview *loopview){
 	g_return_if_fail (loopview != NULL);
 	g_return_if_fail (GTK_IS_LOOPVIEW (loopview));
 
-	gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);
+	if (loopview->looperdata->buf)
+		peakview_calculate(loopview->peakview,0, loopview->zoomend);
+	else gtk_loopview_update_area(loopview,0,loopview->npeaks);
+/*	gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);*/
 /*	gtk_loopview_update (loopview);*/
 }
 
@@ -331,7 +348,7 @@ void gtk_loopview_set_play_pos (GtkLoopview *loopview, gdouble playpos){
 			t2 = loopview->gplaypos;
 			loopview->gplaypos = temp;
 /*			printf ("playcursor: %d -> %d\n",(int)t2,(int)temp);*/
-			if ((loopview->gplaypos - t2) < 3){
+			if (abs(loopview->gplaypos - t2) < 3){
 				gtk_loopview_update_area(loopview,t2,loopview->gplaypos);
 			}else{
 				gtk_loopview_update_area(loopview,t2,t2);
@@ -354,9 +371,11 @@ void gtk_loopview_set_rec_pos (GtkLoopview *loopview, gdouble recpos){
 		if (temp != loopview->grecpos){
 			t2 = loopview->grecpos;
 			loopview->grecpos = temp;
-			if ((loopview->grecpos - t2) < 3){
+			if (abs(loopview->grecpos - t2) < 3){
+/*				printf ("recpos update: %ld - %ld\n",t2,loopview->grecpos);*/
 				gtk_loopview_update_area(loopview,t2,loopview->grecpos);
 			}else{
+/*				printf ("recpos update: %ld , %ld\n",t2,loopview->grecpos);*/
 				gtk_loopview_update_area(loopview,t2,t2);
 				gtk_loopview_update_area(loopview,loopview->grecpos,loopview->grecpos);
 			}
@@ -372,7 +391,9 @@ void gtk_loopview_zoom_to_playselection (GtkLoopview *loopview){
 		loopview->gloopend = gtk_loopview_sample2graphical(loopview,loopview->looperdata->loopend);
 		loopview->grecstart = gtk_loopview_sample2graphical(loopview,loopview->looperdata->recstart);
 		loopview->grecend = gtk_loopview_sample2graphical(loopview,loopview->looperdata->recend);
-		gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);
+/*		gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);*/
+		peakview_set_area(loopview->peakview,loopview->zoomstart, loopview->zoomend, loopview->npeaks);
+		peakview_calculate(loopview->peakview,loopview->zoomstart, loopview->zoomend);
 	}
 }
 
@@ -384,7 +405,9 @@ void gtk_loopview_zoom_to_recselection (GtkLoopview *loopview){
 		loopview->gloopend = gtk_loopview_sample2graphical(loopview,loopview->looperdata->loopend);
 		loopview->grecstart = gtk_loopview_sample2graphical(loopview,loopview->looperdata->recstart);
 		loopview->grecend = gtk_loopview_sample2graphical(loopview,loopview->looperdata->recend);
-		gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);
+/*		gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);*/
+		peakview_set_area(loopview->peakview,loopview->zoomstart, loopview->zoomend, loopview->npeaks);
+                peakview_calculate(loopview->peakview,loopview->zoomstart, loopview->zoomend);
 	}
 }
 
@@ -394,18 +417,21 @@ void gtk_loopview_zoom_to_overview (GtkLoopview *loopview){
 		loopview->zoomend = 0;
 		if (loopview->looperdata)
 			if (loopview->looperdata->buf)
-				loopview->zoomend = loopview->looperdata->buf->size;
+				loopview->zoomend = buffer_get_size(loopview->looperdata->buf);
 		loopview->gloopstart = gtk_loopview_sample2graphical(loopview,loopview->looperdata->loopstart);
 		loopview->gloopend = gtk_loopview_sample2graphical(loopview,loopview->looperdata->loopend);
 		loopview->grecstart = gtk_loopview_sample2graphical(loopview,loopview->looperdata->recstart);
 		loopview->grecend = gtk_loopview_sample2graphical(loopview,loopview->looperdata->recend);
-		gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);
+/*		gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);*/
+		peakview_set_area(loopview->peakview,loopview->zoomstart, loopview->zoomend, loopview->npeaks);
+                peakview_calculate(loopview->peakview,loopview->zoomstart, loopview->zoomend);
 	}
 }
 
 void gtk_loopview_set_zoom (GtkLoopview *loopview, gdouble zoomstart, gdouble zoomend){
 	gdouble temp;
 
+/*	printf ("set zoom\n");*/
 	if (zoomend < zoomstart){
 		temp = zoomend;
 		zoomend = zoomstart;
@@ -416,11 +442,11 @@ void gtk_loopview_set_zoom (GtkLoopview *loopview, gdouble zoomstart, gdouble zo
 		if ((loopview->zoomstart == zoomstart) && (loopview->zoomend == zoomend)) return;
 		if (loopview->looperdata->buf){
 			if (zoomend < 0) zoomend = 0;
-			if (zoomend > loopview->looperdata->buf->size) 
-				zoomend = loopview->looperdata->buf->size;
+			if (zoomend > buffer_get_size(loopview->looperdata->buf)) 
+				zoomend = buffer_get_size(loopview->looperdata->buf);
 			if (zoomstart < 0) zoomstart = 0;
-			if (zoomstart > loopview->looperdata->buf->size) 
-				zoomstart = loopview->looperdata->buf->size;
+			if (zoomstart > buffer_get_size(loopview->looperdata->buf)) 
+				zoomstart = buffer_get_size(loopview->looperdata->buf);
 
 			if (zoomstart < zoomend){					
 				loopview->zoomstart = zoomstart;
@@ -429,7 +455,9 @@ void gtk_loopview_set_zoom (GtkLoopview *loopview, gdouble zoomstart, gdouble zo
 				loopview->gloopend = gtk_loopview_sample2graphical(loopview,loopview->looperdata->loopend);
 				loopview->grecstart = gtk_loopview_sample2graphical(loopview,loopview->looperdata->recstart);
 				loopview->grecend = gtk_loopview_sample2graphical(loopview,loopview->looperdata->recend);
-				gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);
+/*				gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);*/
+				peakview_set_area(loopview->peakview,loopview->zoomstart, loopview->zoomend, loopview->npeaks);
+                		peakview_calculate(loopview->peakview,loopview->zoomstart, loopview->zoomend);
 			}
 		}
 	}
@@ -519,6 +547,26 @@ gdouble gtk_loopview_get_bpm (GtkLoopview *loopview){
 	return loopview->bpm;
 }
 
+void gtk_loopview_set_gridoffset (GtkLoopview *loopview, guint32 offset){
+	if (loopview->looperdata){
+		if (loopview->looperdata->buf){
+			if(offset <= buffer_get_size(loopview->looperdata->buf))
+				loopview->gridoffset = offset;
+			else loopview->gridoffset = buffer_get_size(loopview->looperdata->buf);
+/*			printf ("offset:%ld\n",(long)loopview->gridoffset);*/
+			gtk_loopview_update_area (loopview,0,loopview->npeaks);
+		} else loopview->gridoffset = 0;
+	} else loopview->gridoffset = 0;
+}
+
+guint32 gtk_loopview_get_gridoffset (GtkLoopview *loopview){
+	return loopview->gridoffset;
+}
+
+guint32 gtk_loopview_get_gridwidth (GtkLoopview *loopview){
+        return loopview->gridwidth;
+}
+
 void gtk_loopview_set_bpb (GtkLoopview *loopview, gdouble bpb){
 	if (bpb >= 1.){
 		loopview->bpb = bpb;
@@ -545,36 +593,29 @@ void gtk_loopview_update_recend (GtkLoopview *loopview){
 void gtk_loopview_update_cursorposition (GtkLoopview *loopview){
 	g_return_if_fail (loopview != NULL);
 	g_return_if_fail (GTK_IS_LOOPVIEW (loopview));
-	double start = 0;
-	double end = 0;
-	gint32 gstart = 0;
-	gint32 temp;
-	gint32 gend = 0;
+	double start = 0.;
+	double end = 0.;
 	gint x = 0;
 	gint y = 0;
 	GdkModifierType mods;
+	peakdata_t *pd;
 
-	/*if (loopview->lastnup > 200) loopview->lastnup = 0;
-	else*/ 
 	if (loopview->looperdata){
 		if (loopview->looperdata->buf){
-			looperdata_lock(loopview->looperdata);
 			gdk_window_get_pointer (GTK_WIDGET(loopview)->window, &x, &y, &mods);
-
-			while (looperdata_read_once_dirtyblock(loopview->looperdata, &start, &end)){
-				looperdata_unlock(loopview->looperdata);
-				gstart = gtk_loopview_sample2graphical (loopview, start);
-				gend = gtk_loopview_sample2graphical (loopview, end);
-				if (gstart > gend){ temp = gstart; gstart = gend; gend = temp;}
-				if ((gstart < loopview->npeaks) && (gend > 0)){
-					if (gstart < 0){gstart = 0;}
-					if (gend > loopview->npeaks){gend = loopview->npeaks;}
-					gtk_loopview_calc_peaks(loopview, gstart, gend);
-				}
-				g_thread_yield();
-				looperdata_lock(loopview->looperdata);
-			}
+			looperdata_lock(loopview->looperdata);
+			while (looperdata_read_once_dirtyblock(loopview->looperdata, &start, &end))
+				peakview_calculate(loopview->peakview, (long)start, (long)end);
 			looperdata_unlock(loopview->looperdata);
+
+			if (loopview->peakview){
+				peakview_lock(loopview->peakview);
+				while ((pd = peakview_get_next(loopview->peakview))){
+					gtk_loopview_copy_peaks(loopview,pd);
+					peakdata_set_status(pd, PD_STATUS_DEAD);
+				}
+				peakview_unlock(loopview->peakview);
+			}
 			gtk_loopview_set_play_pos (loopview,loopview->looperdata->playpos);
 			gtk_loopview_set_rec_pos (loopview,loopview->looperdata->recpos);
 		}
@@ -663,15 +704,23 @@ static void gtk_loopview_size_allocate (GtkWidget     *widget,
 				loopview->grecend    = loopview->grecend    * allocation->width / loopview->npeaks;
 			}
 
-		       	loopview->Lmin = realloc (loopview->Lmin,sizeof(gint8) * allocation->width);
-		       	loopview->Lmax = realloc (loopview->Lmax,sizeof(gint8) * allocation->width);
-			loopview->Rmin = realloc (loopview->Rmin,sizeof(gint8) * allocation->width);
-			loopview->Rmax = realloc (loopview->Rmax,sizeof(gint8) * allocation->width);
+		       	loopview->Lmin = realloc (loopview->Lmin,sizeof(gint8) * (allocation->width + 1));
+			memset(loopview->Lmin,0,sizeof(gint8) * allocation->width);
+		       	loopview->Lmax = realloc (loopview->Lmax,sizeof(gint8) * (allocation->width + 1));
+			memset(loopview->Lmax,0,sizeof(gint8) * allocation->width);
+			loopview->Rmin = realloc (loopview->Rmin,sizeof(gint8) * (allocation->width + 1));
+			memset(loopview->Rmin,0,sizeof(gint8) * allocation->width);
+			loopview->Rmax = realloc (loopview->Rmax,sizeof(gint8) * (allocation->width + 1));
+			memset(loopview->Rmax,0,sizeof(gint8) * allocation->width);
 
 			loopview->npeaks = allocation->width;
 			loopview->width = allocation->width;
 			loopview->height = allocation->height;
-			gtk_loopview_calc_peaks(loopview,0,allocation->width);
+/*			gtk_loopview_calc_peaks(loopview,0,allocation->width);*/
+			peakview_set_area(loopview->peakview,loopview->zoomstart, loopview->zoomend, allocation->width);
+			peakview_calculate(loopview->peakview,loopview->zoomstart, loopview->zoomend);
+			if (!loopview->peakview)
+				gtk_loopview_update_area(loopview,0,allocation->width);
 		}else{
 			gtk_loopview_update_area(loopview,0,allocation->width);
 		}
@@ -691,11 +740,11 @@ void gtk_loopview_update_column (GtkLoopview *loopview, int i){
 		gdk_draw_line (loopview->pixmap,loopview->fgc,loopview->gplaypos,1,
 			loopview->gplaypos,loopview->height);
 	}else{
-		if (loopview->showgrid){	
-			diff = gtk_loopview_graphical2sample(loopview,i) / loopview->gridwidth;
-			diff = gtk_loopview_sample2graphical(loopview,(gint)(diff + .5) * loopview->gridwidth);
-			diff = fabs(diff - i);
-		}
+		if (loopview->showgrid){        
+                        diff = (gtk_loopview_graphical2sample(loopview,i) - loopview->gridoffset) / loopview->gridwidth;
+                        diff = gtk_loopview_sample2graphical(loopview,(gint)(diff + .5) * loopview->gridwidth + loopview->gridoffset);
+                        diff = fabs(diff - i);
+                }
 
 		if ((loopview->showgrid) && (diff < .5)){
 			if (((int)((float)gtk_loopview_graphical2sample(loopview,i) / (float)loopview->gridwidth + .5) 
@@ -742,7 +791,7 @@ void gtk_loopview_update_column (GtkLoopview *loopview, int i){
 			i,loopview->Lmid + (*(loopview->Lmin + i) * loopview->Hmax) / 127,
 			i,loopview->Lmid + (*(loopview->Lmax + i) * loopview->Hmax) / 127);
 	
-		if (loopview->looperdata->buf->channels > 1)
+		if (buffer_get_channels(loopview->looperdata->buf) > 1)
 			gdk_draw_line (loopview->pixmap,loopview->fgc,
 				i,loopview->Rmid + (*(loopview->Rmin + i) * loopview->Hmax) / 127,
 				i,loopview->Rmid + (*(loopview->Rmax + i) * loopview->Hmax) / 127);
@@ -785,7 +834,7 @@ void gtk_loopview_update_area(GtkLoopview *loopview, gint32 startx, gint32 endx)
 				loopview->zoomstart + loopview->gridwidth);
 */
 
-		if (loopview->looperdata->buf->channels > 1) {
+		if (buffer_get_channels(loopview->looperdata->buf) > 1) {
 			loopview->Lmid = loopview->height / 4;
 			loopview->Rmid = loopview->height * 3 / 4;
 			loopview->Hmax = (loopview->height / 4 ) - 2;
@@ -811,7 +860,7 @@ void gtk_loopview_update_area(GtkLoopview *loopview, gint32 startx, gint32 endx)
 		}
 	
 		/* middle line */
-		if (loopview->looperdata->buf->channels > 1)
+		if (buffer_get_channels(loopview->looperdata->buf) > 1)
 			gdk_draw_line (loopview->pixmap,
 				widget->style->fg_gc[widget->state],
 				1,loopview->height / 2,
@@ -832,9 +881,11 @@ void gtk_loopview_update_area(GtkLoopview *loopview, gint32 startx, gint32 endx)
 	gtk_widget_draw (GTK_WIDGET(loopview),&update_rect);
 }
 
+#if 0
 void gtk_loopview_calc_peaks(GtkLoopview *loopview,guint32 startx, guint32 endx){
 	float Lval;
 	float Rval;
+	float drift;
 	gint8 *Lminp = NULL;
 	gint8 *Rminp = NULL;
 	gint8 *Lmaxp = NULL;
@@ -848,7 +899,7 @@ void gtk_loopview_calc_peaks(GtkLoopview *loopview,guint32 startx, guint32 endx)
 /*	printf ("calc peaks, ");
 	printf ("height:%ld\n",(long)loopview->height);*/
 
-
+	
 	if (loopview->npeaks < 1) return;
 	if  (loopview->looperdata == NULL){
 		 gtk_loopview_update_area(loopview, startx, endx); 
@@ -859,6 +910,9 @@ void gtk_loopview_calc_peaks(GtkLoopview *loopview,guint32 startx, guint32 endx)
 		return;
 	}
 
+	drift = buffer_get_sampleratefactor(loopview->looperdata->buf,
+				loopview->looperdata->samplerate);	
+
 	if (endx > loopview->npeaks){endx = loopview->npeaks;}
 	if (startx > endx){
 		guint32 tmp = endx;
@@ -873,32 +927,35 @@ void gtk_loopview_calc_peaks(GtkLoopview *loopview,guint32 startx, guint32 endx)
 			steps = (guint32)(samples / loopview->npeaks);
 			Lminp = loopview->Lmin + startx;
 			Lmaxp = loopview->Lmax + startx;
-			if (loopview->looperdata->buf->channels > 1){
+			if (buffer_get_channels(loopview->looperdata->buf) > 1){
 				Rminp = loopview->Rmin + startx;
 				Rmaxp = loopview->Rmax + startx;
 			}
 			for (i = 0; i < peaks; i++){
 				*Lminp = 127; 
 				*Lmaxp = -127;
-				if (loopview->looperdata->buf->channels > 1){
+				if (buffer_get_channels(loopview->looperdata->buf) > 1){
 					*Rminp = 127;
 					*Rmaxp = -127;
 				}
 				for (j = 0; j < steps; j++){
-					Lval = buffer_readsample(loopview->looperdata->buf,0,loopview->zoomstart + 
-						(startx + i) * samples / loopview->npeaks + j) * 127.;
+					float pos = loopview->zoomstart;
+					pos += (startx + i) * (long)samples / (long)loopview->npeaks + j;
+					pos *= drift;
+					Lval = buffer_readsample_interpolated(loopview->looperdata->buf,1, 
+								pos, loopview->buffer_group) * 127.;
 					if ((gint8)Lval < *Lminp ){*Lminp = (gint8)Lval;}
 					if ((gint8)Lval > *Lmaxp ){*Lmaxp = (gint8)Lval;}
-					if (loopview->looperdata->buf->channels > 1){
-					Rval = buffer_readsample(loopview->looperdata->buf,1,loopview->zoomstart +
-						(startx + i) * samples / loopview->npeaks + j) * 127.;
+					if (buffer_get_channels(loopview->looperdata->buf) > 1){
+						Rval = buffer_readsample_interpolated(loopview->looperdata->buf,2, 
+								pos, loopview->buffer_group) * 127.;
 						if ((gint8)Rval < *Rminp ){*Rminp = (gint8)Rval;}
 						if ((gint8)Rval > *Rmaxp ){*Rmaxp = (gint8)Rval;}
 					}
 				}
 
 				Lminp++; Lmaxp++; 
-				if (loopview->looperdata->buf->channels > 1){
+				if (buffer_get_channels(loopview->looperdata->buf) > 1){
 					Rminp++; Rmaxp++;
 				}
 			}	
@@ -908,6 +965,24 @@ void gtk_loopview_calc_peaks(GtkLoopview *loopview,guint32 startx, guint32 endx)
 	}
 /*	printf ("calc peaks done\n");*/
 }
+#endif
+
+void gtk_loopview_copy_peaks(GtkLoopview *loopview, peakdata_t *pd){
+	long i;
+
+/*	printf ("copy: %p. done:%d\n",pd,pd->done);*/
+	/* LATER: use mmap */
+	for (i = 0; i < (pd->pend - pd->pstart);i++){
+		*(loopview->Lmin + i + pd->pstart) = (gint8)*(pd->Lmin + i);
+		*(loopview->Lmax + i + pd->pstart) = (gint8)*(pd->Lmax + i);
+		if (buffer_get_channels(loopview->looperdata->buf) > 1){
+			*(loopview->Rmin + i + pd->pstart) = (gint8)*(pd->Rmin + i);
+                	*(loopview->Rmax + i + pd->pstart) = (gint8)*(pd->Rmax + i);
+		}
+	}
+/*	printf ("updated:...%ld - %ld\n",pd->pstart, (pd->pstart + i));*/
+	gtk_loopview_update_area(loopview, pd->pstart, pd->pend);
+}
 
 static gboolean gtk_loopview_expose( GtkWidget *widget, GdkEventExpose *event){
   	GtkLoopview *loopview = GTK_LOOPVIEW (widget);
@@ -958,10 +1033,11 @@ static gdouble snap2grid(GtkLoopview *loopview, gdouble x){
 
 	if (loopview->snap2grid == TRUE){
 		/* equivalent of gdelta pixels */
-		delta = gtk_loopview_graphical2sample(loopview, loopview->gloopstart + gdelta) -
-			gtk_loopview_graphical2sample(loopview, loopview->gloopstart); 
-		nearest = (gint)(x / loopview->gridwidth + .5) * loopview->gridwidth;
-		diff = x - nearest;
+                delta = gtk_loopview_graphical2sample(loopview, loopview->gloopstart + gdelta) -
+                        gtk_loopview_graphical2sample(loopview, loopview->gloopstart); 
+                nearest = (gint)((x - loopview->gridoffset) / loopview->gridwidth + .5) * loopview->gridwidth + loopview->gridoffset;
+/*		nearest = (gint)(x / loopview->gridwidth + .5) * loopview->gridwidth; */
+                diff = x - nearest;
 		if (fabs(diff) < delta)	 x = nearest;
 /*		printf ("x:%lf, nearest:%lf, diff:%lf, delta:%lf\n",
 			(double)x,(double)nearest,(double)nearest,(double)delta);*/
@@ -975,12 +1051,12 @@ static void gtk_loopview_set_loopstartend(GtkLoopview *loopview, gdouble temps,
 		looperdata_get_loopstart(loopview->looperdata),looperdata_get_loopend(loopview->looperdata),temps,tempe);
 */
 	if (temps < .0){temps = 0.;}
-	if (temps > loopview->looperdata->buf->size){
-		temps = loopview->looperdata->buf->size;
+	if (temps > buffer_get_size(loopview->looperdata->buf)){
+		temps = buffer_get_size(loopview->looperdata->buf);
 	}
 	if (tempe < .0){tempe = 0.;}
-	if (tempe > loopview->looperdata->buf->size){
-		tempe = loopview->looperdata->buf->size;
+	if (tempe > buffer_get_size(loopview->looperdata->buf)){
+		tempe = buffer_get_size(loopview->looperdata->buf);
 	}
 	if (temps > tempe){
 		gdouble temp = temps;
@@ -995,12 +1071,12 @@ static void gtk_loopview_set_loopstartend(GtkLoopview *loopview, gdouble temps,
 
 static void gtk_loopview_set_recstartend(GtkLoopview *loopview, gdouble temps, gdouble tempe){
 	if (temps < .0){temps = 0.;}
-	if (temps > loopview->looperdata->buf->size){
-		temps = loopview->looperdata->buf->size;
+	if (temps > buffer_get_size(loopview->looperdata->buf)){
+		temps = buffer_get_size(loopview->looperdata->buf);
 	}
 	if (tempe < .0){tempe = 0.;}
-	if (tempe > loopview->looperdata->buf->size){
-		tempe = loopview->looperdata->buf->size;
+	if (tempe > buffer_get_size(loopview->looperdata->buf)){
+		tempe = buffer_get_size(loopview->looperdata->buf);
 	}
 	if (temps > tempe){
 		gdouble temp = temps;
@@ -1233,8 +1309,8 @@ static gboolean gtk_loopview_button_release( GtkWidget      *widget,
 /*				printf ("temps:%f, tempe:%f\n",(float)temps, (float)tempe);*/
 				if (temps < 0){temps = 0;}
 				if (loopview->looperdata->buf){
-					if (tempe > loopview->looperdata->buf->size){
-						tempe = loopview->looperdata->buf->size;
+					if (tempe > buffer_get_size(loopview->looperdata->buf)){
+						tempe = buffer_get_size(loopview->looperdata->buf);
 					}
 				}
 			}
@@ -1246,10 +1322,12 @@ static gboolean gtk_loopview_button_release( GtkWidget      *widget,
 				loopview->grecstart  = gtk_loopview_sample2graphical(loopview,loopview->looperdata->recstart);
 				loopview->grecend    = gtk_loopview_sample2graphical(loopview,loopview->looperdata->recend);
 /*				gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);*/
+				peakview_set_area(loopview->peakview,loopview->zoomstart, loopview->zoomend, loopview->npeaks);
 			}
 			loopview->gzoomstart = 0;
 			loopview->gzoomend = 0;
-			gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);
+/*			gtk_loopview_calc_peaks(loopview,0,loopview->npeaks);*/
+			peakview_calculate(loopview->peakview, loopview->zoomstart, loopview->zoomend);
 /*			gtk_widget_draw (GTK_WIDGET(loopview),NULL);*/
 		}
 
