@@ -24,6 +24,11 @@
 #include "../../common/midi.h"
 #endif
 
+#ifdef HAVE_OSC
+#include "../../common/osc.h"
+#endif
+
+
 #define DIR_MODE (S_IRWXU|S_IRGRP)
 
 /* global widgets: */
@@ -89,6 +94,10 @@ jack_info_t *jack_info;
 midi_info_t *midi_info;
 #endif
 
+#ifdef HAVE_OSC
+osc_info_t *osc_info;
+#endif
+
 /*------- EVENT FUNCTIONS -------*/
 static gboolean delete_event( GtkWidget *widget,
 			      GdkEvent  *event,
@@ -338,10 +347,16 @@ gint main_idlefunc(gpointer data){
 
 	if (countdown) countdown--;	
 	else if (loadfilestate == 1){
+                gint x,y,w,h;
+
 /*		printf ("stage 1\n");*/
+                config_get_geometry(ch, &x,&y,&w,&h);
+                gtk_window_resize((GtkWindow*)window,w,h);
+                gtk_window_move((GtkWindow*)window,x,y);
+
 		while ((ti = config_next_buffer(ch)) != 0){
 			if (bufferindexcounter < ti) bufferindexcounter = ti;
-		       	buf = buffer_new(ti);
+		       	buf = buffer_new(ti,jack_info->samplerate);
 			config_read_buffer (ch,buf);
 /*			printf ("buffer nr:%d created\n",ti);*/
 			speciallist_append(buffer_list,buf);
@@ -369,10 +384,8 @@ gint main_idlefunc(gpointer data){
 		loadfilestate++;
 		countdown = 5; /* wait a while to settle widgets */
 	}else if (loadfilestate == 3){
-		gint lastwidth = 0;
-		gint lastheight = 0;
-
 /*		printf ("stage 3\n");*/
+
 		tmpptr = g_list_first (looperlist->listptr);
 		while (tmpptr){
 			looper = (looper_gui_t*)tmpptr->data;
@@ -381,12 +394,11 @@ gint main_idlefunc(gpointer data){
 					config_read_looper (ch,looper);
 			tmpptr = g_list_next(tmpptr);
 		}
-		close_config_reader(ch);
 		gtk_widget_destroy (waitdialog);
 		loadfilestate = 0;
-		gtk_window_get_size ((GtkWindow*)window,&lastwidth,&lastheight);
-		gtk_window_resize((GtkWindow*)window,lastwidth,1);
 		set_status("project loaded");
+
+		close_config_reader(ch);
 	}
 
 	/* reset statusbar if inactive for 2+ seconds */
@@ -435,55 +447,86 @@ gint main_idlefunc(gpointer data){
 
 /*-------GUI CREATE FUNCTIONS----------*/
 
-static void load_buffer(gpointer data){
+static char* loadbufferdialog(char* filename){
 	GtkWidget *gfc;
-	GtkWidget *sfp;
-	buffer_info_t *buf = NULL;
-	char* pathend = NULL;
-	struct stat     fstat;
+        GtkWidget *sfp;
+        struct stat     fstat;
 
-	gfc =  gtk_file_chooser_dialog_new("select a soundfile for the new buffer",
-			GTK_WINDOW(window),
-			GTK_FILE_CHOOSER_ACTION_OPEN,
-			GTK_STOCK_CANCEL,
-			GTK_RESPONSE_CANCEL,
-			GTK_STOCK_OPEN,
-			GTK_RESPONSE_ACCEPT,
-					NULL);
-	sfp = sfpreview_new();
-	g_signal_connect (G_OBJECT (gfc), "update-preview",
-			G_CALLBACK (sfpreview_update), (gpointer) sfp);
-	gtk_file_chooser_set_use_preview_label(GTK_FILE_CHOOSER (gfc),FALSE);
-	gtk_file_chooser_set_preview_widget(GTK_FILE_CHOOSER (gfc),GTK_WIDGET(sfp));
-	gtk_file_chooser_set_show_hidden(GTK_FILE_CHOOSER (gfc),TRUE);
-	if (lastpath) gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (gfc),lastpath);
+        gfc =  gtk_file_chooser_dialog_new("select a soundfile for the new buffer",
+                        GTK_WINDOW(window),
+                        GTK_FILE_CHOOSER_ACTION_OPEN,
+                        GTK_STOCK_CANCEL,
+                        GTK_RESPONSE_CANCEL,
+                        GTK_STOCK_OPEN,
+                        GTK_RESPONSE_ACCEPT,
+                                        NULL);
+        sfp = sfpreview_new();
+        g_signal_connect (G_OBJECT (gfc), "update-preview",
+                        G_CALLBACK (sfpreview_update), (gpointer) sfp);
+        gtk_file_chooser_set_use_preview_label(GTK_FILE_CHOOSER (gfc),FALSE);
+        gtk_file_chooser_set_preview_widget(GTK_FILE_CHOOSER (gfc),GTK_WIDGET(sfp));
+        gtk_file_chooser_set_show_hidden(GTK_FILE_CHOOSER (gfc),TRUE);
+        if (lastpath) gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (gfc),lastpath);
 
-	/* LATER: add a preview-function to listen to files in advance, and a GtkFileFilter */
+        /* LATER: add a preview-function to listen to files in advance, and a GtkFileFilter */
 
-	if (gtk_dialog_run (GTK_DIALOG (gfc)) == GTK_RESPONSE_ACCEPT){
-		char *filename;
-		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (gfc));
-
-		if (stat (filename, &fstat))
-			printf ("file not found!\n");
-		else{
-			pathend = strrchr (filename, '/');
-			if (pathend){
-				lastpath = realloc (lastpath,sizeof(char) * (pathend - filename + 2));
-				strncpy (lastpath, filename, (pathend - filename + 1));
-				strncpy ((char*)(lastpath + (long)(pathend - filename + 1)),"\0",1);
-/*				printf ("last path is now:%s\n",pathend);*/
-			}
-			buf = buffer_new(++bufferindexcounter);
-			if (jack_info->samplerate)
-				buffer_loadfile(buf,(char*)filename,jack_info->samplerate);
+        if (gtk_dialog_run (GTK_DIALOG (gfc)) == GTK_RESPONSE_ACCEPT){
+                filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (gfc));
 		
-			speciallist_append(buffer_list,buf);	
+                if (stat (filename, &fstat)){
+                        printf ("file not found!\n");
+			if(filename) free(filename);
+			filename = NULL;
+                }
+        }
+        gtk_widget_destroy (sfp);
+        gtk_widget_destroy (gfc);
+
+	return filename;
+}
+
+static void load_buffer(gpointer data){
+	char *filename = NULL;
+	buffer_info_t *buf = NULL;
+        char* pathend = NULL;
+
+	filename = loadbufferdialog (filename);
+	if (filename){
+		pathend = strrchr (filename, '/');
+		if (pathend){
+			lastpath = realloc (lastpath,sizeof(char) * (pathend - filename + 2));
+			strncpy (lastpath, filename, (pathend - filename + 1));
+			strncpy ((char*)(lastpath + (long)(pathend - filename + 1)),"\0",1);
 		}
-		g_free (filename);
-	}
-	gtk_widget_destroy (sfp);
-	gtk_widget_destroy (gfc);
+		buf = buffer_new(++bufferindexcounter,jack_info->samplerate);
+		if (jack_info->samplerate){	
+			buffer_loadfile(buf,(char*)filename,jack_info->samplerate);
+		}
+		speciallist_append(buffer_list,buf);	
+	} else set_status("loading failed");
+	g_free (filename);
+}
+
+static void open_discstream(gpointer data){
+        char *filename = NULL; 
+	buffer_info_t *buf = NULL;
+        char* pathend = NULL;
+
+        filename = loadbufferdialog (filename);
+        if (filename){
+                pathend = strrchr (filename, '/');
+                if (pathend){
+                        lastpath = realloc (lastpath,sizeof(char) * (pathend - filename + 2));
+                        strncpy (lastpath, filename, (pathend - filename + 1));
+                        strncpy ((char*)(lastpath + (long)(pathend - filename + 1)),"\0",1);
+                }       
+                buf = buffer_new(++bufferindexcounter,jack_info->samplerate);
+                if (jack_info->samplerate){
+                        buffer_open_discstream(buf,(char*)filename,jack_info->samplerate);
+		}
+                speciallist_append(buffer_list,buf);    
+        } else set_status("loading failed");
+        g_free (filename);
 }
 
 static void open_project (char *filename){
@@ -553,7 +596,7 @@ static void about_callback(gpointer data){
                                 GTK_MESSAGE_INFO,
                                 GTK_BUTTONS_CLOSE,
                                	"kluppe looper is developed by dieter kovacic\n"
-				"http://dieb13.klingt.org/content/projects/kluppe.html\n"
+				"http://kluppe.klingt.org\n"
                         	"contact: kluppe" "@" "klingt.org.\n");
         response = gtk_dialog_run (GTK_DIALOG (dialog));
                         
@@ -583,11 +626,43 @@ static void open_project_menucallback(gpointer data){
 	gtk_widget_destroy (gfc);
 }
 
+static void new_project(){
+        close_all_loopers();
+        close_all_buffers();
+        looperindexcounter = 0;
+        bufferindexcounter = 0;
+        if (projectfilename){
+                free(projectfilename);
+                projectfilename = NULL;
+        }
+}
+
+static void new_project_menucallback(gpointer data){
+	GtkWidget *dialog;
+        gint response;
+
+        dialog = gtk_message_dialog_new (GTK_WINDOW(window),
+                                GTK_DIALOG_DESTROY_WITH_PARENT,
+                                GTK_MESSAGE_QUESTION,
+				GTK_BUTTONS_YES_NO,
+                                "do you really want to close\n"
+                                "this project and remove all buffers?\n");
+        response = gtk_dialog_run (GTK_DIALOG (dialog));
+                        
+        gtk_widget_destroy (dialog);
+	if (response == GTK_RESPONSE_YES){
+		new_project();
+		set_status("project colsed");
+	}else set_status("not coling project");
+	
+}
+
 static void save_project_as_menucallback(gpointer data){
 	GtkWidget *gfc;
 	char* pathend = NULL;
 	struct stat fstat;
 	int  ret;
+	gint x,y,w,h;
        	gint response = 0;
  
 	gfc =  gtk_file_chooser_dialog_new("select a projectfilename for saving",
@@ -613,7 +688,9 @@ static void save_project_as_menucallback(gpointer data){
 		if (response == GTK_RESPONSE_ACCEPT){
 
 			/* gtk_window_get_position () to save position too ??? */
-			ret = save_config (looperlist->listptr, buffer_list, lastpath, lastconfigfilepath, filename);
+			gtk_window_get_size(GTK_WINDOW (window),&w,&h);
+			gtk_window_get_position(GTK_WINDOW (window),&x,&y);
+			ret = save_config (looperlist->listptr, buffer_list, lastpath, lastconfigfilepath, filename,x,y,w,h);
 			if (ret < 0)	set_status("saving failed !");
 			else {
 				projectfilename = realloc (projectfilename, sizeof(char) * (strlen(filename) +1));
@@ -638,9 +715,12 @@ static void save_project_as_menucallback(gpointer data){
 
 static void save_project_menucallback(gpointer data){
 	int ret = 0;
+	gint x,y,w,h;
 
+	gtk_window_get_size(GTK_WINDOW (window),&w,&h);
+	gtk_window_get_position(GTK_WINDOW (window),&x,&y);
 	if (projectfilename)
-		ret = save_config (looperlist->listptr, buffer_list, lastpath, lastconfigfilepath, projectfilename);
+		ret = save_config (looperlist->listptr, buffer_list, lastpath, lastconfigfilepath, projectfilename,x,y,w,h);
 	else save_project_as_menucallback(data);
 
 	if (ret < 0)  set_status("saving failed !");
@@ -660,7 +740,7 @@ static void create_empty_buffer(gpointer data){
 	stereo = gtk_check_button_new_with_label ("stereo");
 	txt = realloc(txt,sizeof(char) * 256);
 	memset(txt,0,sizeof(char) * 255);
-	sprintf (txt,"buffer nr. %d",++bufferindexcounter);
+	sprintf (txt,"MB: buffer nr. %d",++bufferindexcounter);
 	nameline = gtk_entry_new();
 	gtk_entry_set_max_length((GtkEntry*)nameline,255);
 	gtk_entry_set_text((GtkEntry*)nameline,(gchar*)txt);
@@ -687,7 +767,7 @@ static void create_empty_buffer(gpointer data){
 	gtk_widget_show_all (dialog);
 
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT){
-		buf = buffer_new(bufferindexcounter);
+		buf = buffer_new(bufferindexcounter,jack_info->samplerate);
 		if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(stereo)))
 			buffer_set_channels(buf,1);
 		else
@@ -697,8 +777,8 @@ static void create_empty_buffer(gpointer data){
 		if (jack_info->samplerate)
 			buffer_resize (buf,
 				(long)(gtk_spin_button_get_value ((GtkSpinButton*)spin) * jack_info->samplerate));
-		if (buf->size){
-/*			printf ("entry:%s.\n",(char*)gtk_entry_get_text((GtkEntry*)nameline));*/
+/*		printf ("i'm here:%ld\n",buffer_get_size(buf));*/
+		if (buffer_get_size(buf)){
 			buffer_set_shortname(buf,(char*)gtk_entry_get_text((GtkEntry*)nameline));
 			buf->pending = BUFFER_STATUS_READY;
 		}else{
@@ -747,6 +827,11 @@ static void create_mainlayout(){
 	gtk_box_pack_start (GTK_BOX (footerbox),newbufferbutton,FALSE, FALSE, 0);
 	gtk_widget_show (newbufferbutton);
 
+	newbufferbutton = gtk_button_new_with_label (" new discstream ");
+        g_signal_connect (G_OBJECT(newbufferbutton), "clicked", G_CALLBACK(open_discstream), NULL);
+        gtk_box_pack_start (GTK_BOX (footerbox),newbufferbutton,FALSE, FALSE, 0);
+        gtk_widget_show (newbufferbutton);
+
 	newemptybufferbutton  = gtk_button_new_with_label (" new empty buffer ");
 	g_signal_connect (G_OBJECT(newemptybufferbutton), "clicked", G_CALLBACK(create_empty_buffer), NULL);
 	gtk_box_pack_start (GTK_BOX (footerbox),newemptybufferbutton,FALSE, FALSE, 0);
@@ -769,6 +854,12 @@ static void create_mainlayout(){
 	/* top menu */
 	menu = gtk_menu_new ();
 
+	menu_items = gtk_menu_item_new_with_label ("new empty project");
+        gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_items);
+        g_signal_connect_swapped (G_OBJECT (menu_items), "activate",
+                                      G_CALLBACK (new_project_menucallback),NULL);
+        gtk_widget_show (menu_items);
+
 	menu_items = gtk_menu_item_new_with_label ("open project");
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_items);
 	g_signal_connect_swapped (G_OBJECT (menu_items), "activate",
@@ -812,21 +903,25 @@ static void create_mainlayout(){
 
         menu_items = gtk_menu_item_new_with_label ("new looper");
         gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_items);
-
         g_signal_connect_swapped (G_OBJECT (menu_items), "activate",
                                       G_CALLBACK (create_looper),
                                       NULL);
         gtk_widget_show (menu_items);
 
-        
         menu_items = gtk_menu_item_new_with_label ("new buffer from file");
         gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_items);
-        
         g_signal_connect_swapped (G_OBJECT (menu_items), "activate",
                                       G_CALLBACK (load_buffer),
                                       NULL);
         gtk_widget_show (menu_items);
 
+        menu_items = gtk_menu_item_new_with_label ("new discstream");
+        gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_items);
+        g_signal_connect_swapped (G_OBJECT (menu_items), "activate",
+                                      G_CALLBACK (open_discstream),
+                                      NULL);
+        gtk_widget_show (menu_items);
+
         menu_items = gtk_menu_item_new_with_label ("new empty buffer");
         gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_items);
         g_signal_connect_swapped (G_OBJECT (menu_items), "activate",
@@ -906,13 +1001,17 @@ int main(int argc, char *argv[])
 	looperdatalist = speciallist_new();
 
 	/* start the loadbuffer thread */
-	buffer_loadthread_launch(buffer_list);
+	buffer_discthread_launch(buffer_list);
 
 #ifdef HAVE_ALSA
         /* create midi thread: */
         midi_info = midi_info_new(looperdatalist);
 /*        printf ("midi thread-create done\n"); */
 #endif
+
+#ifdef HAVE_OSC
+	osc_info = osc_info_new(looperdatalist);
+#endif
 	
 	/* start the jack threads */
         jack_info = mixer_launch(looperdatalist);
