@@ -115,10 +115,11 @@ static void looper_normalize_callback (GtkWidget *normbutton, gpointer data ){
         float factor;
 
         if (looper->data->buf){
-                factor =  buffer_analyze (looper->data->buf, 0 , 0);
+                factor =  buffer_analyze (looper->data->buf, looper->gid, 0 , 0);
 		if (factor != 0){
 			factor = 1. / factor;
-                	buffer_normalize (looper->data->buf, factor, 0, 0);
+/*                	buffer_normalize (looper->data->buf, factor, 0, 0);*/
+			buffer_set_volume_factor(looper->data->buf, factor);
 		}
         }
 }
@@ -128,7 +129,7 @@ static void looper_normalize_playselection_callback (GtkWidget *normbutton, gpoi
         float factor;
 
         if (looper->data->buf){
-                factor =  buffer_analyze (looper->data->buf, 
+                factor =  buffer_analyze (looper->data->buf, looper->gid,
 			looperdata_get_loopstart(looper->data), looperdata_get_loopend(looper->data));
                 if (factor != 0){
                         factor = 1. / factor;
@@ -143,7 +144,7 @@ static void looper_normalize_recselection_callback (GtkWidget *normbutton, gpoin
         float factor;
 
         if (looper->data->buf){
-                factor =  buffer_analyze (looper->data->buf, 
+                factor =  buffer_analyze (looper->data->buf, looper->gid,
 			looperdata_get_recstart(looper->data), looperdata_get_recend(looper->data));
                 if (factor != 0){
                         factor = 1. / factor;
@@ -254,7 +255,7 @@ static void looper_savebuffer2file_callback (GtkWidget *dummy, gpointer data){
 	filename = looper_get_save_filename(GTK_WINDOW(looper->parent));
 	if (filename){
 		ret = buffer_save(looper->data->buf, filename, 
-			looper->samplerate,  0, looper->data->buf->size);
+			looper->samplerate,  0, buffer_get_size(looper->data->buf));
 		if (ret < 1) printf ("error saving file\n");
 		else{
 /*			printf("saved");*/
@@ -332,16 +333,16 @@ static void looper_normbutton_callback (GtkWidget *normbutton,gpointer data ){
 void looper_update_widgets(looper_gui_t *looper){
 	if (!looper->data) return;
 	if (!looper->data->buf) return;
-	if (!looper->data->buf->size) return;
+	if (!buffer_get_size(looper->data->buf)) return;
 
 	if (looper->data->keepabsolute){
                 looper->looplengthblock = 1;
 		gtk_spin_button_set_range((GtkSpinButton*)looper->loopstartspin,
-                	0,samples2speedsecs(looper,looper->data->buf->size));
+                	0,samples2speedsecs(looper,buffer_get_size(looper->data->buf)));
 		gtk_spin_button_set_range((GtkSpinButton*)looper->loopendspin,
-                	0,samples2speedsecs(looper,looper->data->buf->size));
+                	0,samples2speedsecs(looper,buffer_get_size(looper->data->buf)));
 		gtk_spin_button_set_range((GtkSpinButton*)looper->looplengthspin,
-                	0,samples2speedsecs(looper,looper->data->buf->size));
+                	0,samples2speedsecs(looper,buffer_get_size(looper->data->buf)));
 
         	looper->looplengthblock = 1;
 		gtk_spin_button_set_value ((GtkSpinButton*)looper->looplengthspin,
@@ -354,11 +355,11 @@ void looper_update_widgets(looper_gui_t *looper){
 
 		looper->recstartblock = 1;
                 gtk_spin_button_set_range((GtkSpinButton*)looper->recstartspin,
-                        0,samples2speedsecs(looper,looper->data->buf->size));
+                        0,samples2speedsecs(looper,buffer_get_size(looper->data->buf)));
                 gtk_spin_button_set_range((GtkSpinButton*)looper->recendspin,
-                        0,samples2speedsecs(looper,looper->data->buf->size));
+                        0,samples2speedsecs(looper,buffer_get_size(looper->data->buf)));
                 gtk_spin_button_set_range((GtkSpinButton*)looper->reclengthspin,
-                        0,samples2speedsecs(looper,looper->data->buf->size));
+                        0,samples2speedsecs(looper,buffer_get_size(looper->data->buf)));
 
                 looper->reclengthblock = 1;
 		gtk_spin_button_set_value ((GtkSpinButton*)looper->reclengthspin,
@@ -372,11 +373,11 @@ void looper_update_widgets(looper_gui_t *looper){
 	}else{
                 looper->looplengthblock = 1;
 		gtk_spin_button_set_range((GtkSpinButton*)looper->looplengthspin,
-                        0,samples2secs(looper,looper->data->buf->size));
+                        0,samples2secs(looper,buffer_get_size(looper->data->buf)));
 		gtk_spin_button_set_range((GtkSpinButton*)looper->loopstartspin,
-                	0,samples2secs(looper,looper->data->buf->size));
+                	0,samples2secs(looper,buffer_get_size(looper->data->buf)));
         	gtk_spin_button_set_range((GtkSpinButton*)looper->loopendspin,
-                	0,samples2secs(looper,looper->data->buf->size));
+                	0,samples2secs(looper,buffer_get_size(looper->data->buf)));
 
                 looper->looplengthblock = 1;
 		gtk_spin_button_set_value ((GtkSpinButton*)looper->looplengthspin,
@@ -389,11 +390,11 @@ void looper_update_widgets(looper_gui_t *looper){
 
         	looper->reclengthblock = 1;
 		gtk_spin_button_set_range((GtkSpinButton*)looper->reclengthspin,
-                        0,samples2secs(looper,looper->data->buf->size));
+                        0,samples2secs(looper,buffer_get_size(looper->data->buf)));
         	gtk_spin_button_set_range((GtkSpinButton*)looper->recstartspin,
-                	0,samples2secs(looper,looper->data->buf->size));
+                	0,samples2secs(looper,buffer_get_size(looper->data->buf)));
         	gtk_spin_button_set_range((GtkSpinButton*)looper->recendspin,
-                	0,samples2secs(looper,looper->data->buf->size));
+                	0,samples2secs(looper,buffer_get_size(looper->data->buf)));
 
                 looper->reclengthblock = 1;
 		gtk_spin_button_set_value ((GtkSpinButton*)looper->reclengthspin,
@@ -420,7 +421,6 @@ void looper_update_widgets(looper_gui_t *looper){
 	gtk_spin_button_set_value ((GtkSpinButton*)looper->densityspin,
                         looper->data->graindensity);
 	
-
 	gtk_loopview_update_loopstart((GtkLoopview*)looper->loopview);
         gtk_loopview_update_loopend((GtkLoopview*)looper->loopview);
 	gtk_loopview_update_recstart((GtkLoopview*)looper->loopview);
@@ -449,13 +449,25 @@ void looper_set_buffer_by_id (looper_gui_t *looper, int id){
 	if (buf){
 		looperdata_lock(looper->data);
 		new_data = looperdata_new(buf, looper->samplerate, looper->id);
+		looper->gid = buffer_get_group (buf, DS_PRIORITY_RELAXED);
 		if (looper->data){
 			looperdata_copy_settings (new_data, looper->data);
 			looper->data->isplaying = 0;
 			looper->data->isrecording = 0;
+			if (buffer_get_type(buf) == BUFFER_TYPE_DISCSTREAM)
+				looperdata_set_recording(new_data,0);
 		}
 		old_data = looper->data;
 		looper->data = new_data;
+
+		/* try: */
+                /* LATER: replayce with gui */
+/*                printf ("set reverb\n");
+                looperdata_set_reverblength(looper->data,10000);
+                looperdata_set_feedback(looper->data,.9);
+                printf ("done\n");
+*/
+
 		looperdata_unlock(looper->data);
 
 /*		printf ("set buffer:%s\n",looper->data->buf->shortname);*/
@@ -469,7 +481,12 @@ void looper_set_buffer_by_id (looper_gui_t *looper, int id){
 
 		looperdata_free (old_data);
 
+                if (buffer_get_type(buf) == BUFFER_TYPE_DISCSTREAM)
+                        gtk_widget_set_sensitive(GTK_WIDGET(looper->recbutton), FALSE);
+                else    gtk_widget_set_sensitive(GTK_WIDGET(looper->recbutton), TRUE);
+
 		looper_update_widgets(looper);
+		gtk_loopview_update_waveform(GTK_LOOPVIEW(looper->loopview));
 		looper_update_bufferlist(looper,looper->bufferlist);
 	}
 /*	printf ("set_bufferby_id callback done\n");*/
@@ -500,49 +517,59 @@ int looper_update_bufferlist (looper_gui_t* looper, speciallist_t *bufferlist){
                                                 	(gpointer)looper);
         		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
         		gtk_widget_show (menu_item);
-			
-                        menu_item =  gtk_menu_item_new_with_label ("normalize play selection");
-                        g_signal_connect (G_OBJECT (menu_item), "activate",
-                                                        G_CALLBACK (looper_normalize_playselection_callback),
-                                                        (gpointer)looper);
-                        gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-                        gtk_widget_show (menu_item);
+		
+                       	menu_item =  gtk_menu_item_new_with_label ("normalize play selection");
+                       	g_signal_connect (G_OBJECT (menu_item), "activate",
+                                                       	G_CALLBACK (looper_normalize_playselection_callback),
+                                                       	(gpointer)looper);
+                       	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+			if (buffer_get_type(looper->data->buf) == BUFFER_TYPE_DISCSTREAM)
+                               	gtk_widget_set_sensitive(GTK_WIDGET(menu_item), FALSE);
+                       	gtk_widget_show (menu_item);
 
-                        menu_item =  gtk_menu_item_new_with_label ("normalize rec selection");
-                        g_signal_connect (G_OBJECT (menu_item), "activate",
-                                                        G_CALLBACK (looper_normalize_recselection_callback),
-                                                        (gpointer)looper);
-                        gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-                        gtk_widget_show (menu_item);
+                       	menu_item =  gtk_menu_item_new_with_label ("normalize rec selection");
+                       	g_signal_connect (G_OBJECT (menu_item), "activate",
+                                                       	G_CALLBACK (looper_normalize_recselection_callback),
+                                                       	(gpointer)looper);
+                       	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+			if (buffer_get_type(looper->data->buf) == BUFFER_TYPE_DISCSTREAM)
+                               	gtk_widget_set_sensitive(GTK_WIDGET(menu_item), FALSE);
+                       	gtk_widget_show (menu_item);
 
                         menu_item = gtk_separator_menu_item_new();
                         gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
                         gtk_widget_show (menu_item);
 	
-        		menu_item =  gtk_menu_item_new_with_label ("clear active buffer");
-        		g_signal_connect (G_OBJECT (menu_item), "activate",
-                                                	G_CALLBACK (looper_clear_callback),
-                                                	(gpointer)looper);
-        		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-        		gtk_widget_show (menu_item);
+			menu_item =  gtk_menu_item_new_with_label ("clear active buffer");
+       			g_signal_connect (G_OBJECT (menu_item), "activate",
+                                               		G_CALLBACK (looper_clear_callback),
+                                               		(gpointer)looper);
+       			gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+			if (buffer_get_type(looper->data->buf) == BUFFER_TYPE_DISCSTREAM)
+                               	gtk_widget_set_sensitive(GTK_WIDGET(menu_item), FALSE);
+       			gtk_widget_show (menu_item);
 	
-                        menu_item =  gtk_menu_item_new_with_label ("clear play selection");
-                        g_signal_connect (G_OBJECT (menu_item), "activate",
-                                                        G_CALLBACK (looper_clear_playselection_callback),
-                                                        (gpointer)looper);
-                        gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-                        gtk_widget_show (menu_item);
+                       	menu_item =  gtk_menu_item_new_with_label ("clear play selection");
+                       	g_signal_connect (G_OBJECT (menu_item), "activate",
+                                                       	G_CALLBACK (looper_clear_playselection_callback),
+                                                       	(gpointer)looper);
+                       	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+			if (buffer_get_type(looper->data->buf) == BUFFER_TYPE_DISCSTREAM)
+				gtk_widget_set_sensitive(GTK_WIDGET(menu_item), FALSE);
+                       	gtk_widget_show (menu_item);
 
-                        menu_item =  gtk_menu_item_new_with_label ("clear rec selection");
-                        g_signal_connect (G_OBJECT (menu_item), "activate",
-                                                        G_CALLBACK (looper_clear_recselection_callback),
-                                                        (gpointer)looper);
-                        gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-                        gtk_widget_show (menu_item);
-                
-                        menu_item = gtk_separator_menu_item_new();
-                        gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-                        gtk_widget_show (menu_item);
+                       	menu_item =  gtk_menu_item_new_with_label ("clear rec selection");
+                       	g_signal_connect (G_OBJECT (menu_item), "activate",
+                                                       	G_CALLBACK (looper_clear_recselection_callback),
+                                                       	(gpointer)looper);
+                       	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+			if (buffer_get_type(looper->data->buf) == BUFFER_TYPE_DISCSTREAM)
+				gtk_widget_set_sensitive(GTK_WIDGET(menu_item), FALSE);
+                       	gtk_widget_show (menu_item);
+               	
+                       	menu_item = gtk_separator_menu_item_new();
+                       	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+                       	gtk_widget_show (menu_item);
 
 			menu_item =  gtk_menu_item_new_with_label ("save buffer content to file");
                         g_signal_connect (G_OBJECT (menu_item), "activate",
@@ -575,7 +602,7 @@ int looper_update_bufferlist (looper_gui_t* looper, speciallist_t *bufferlist){
 		buf = speciallist_get_first(bufferlist);
 		while(buf){
 			if (buf->status == BUFFER_STATUS_READY){
-				menu_item = gtk_menu_item_new_with_label ((gchar*)buf->shortname);
+				menu_item = gtk_menu_item_new_with_label ((gchar*)buffer_get_shortname(buf));
 /*				gtk_widget_modify_font(menu_item,looper->font);*/
 				gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
 				g_object_set_data (G_OBJECT (menu_item), "kluppe-buffer",&buf->id);
@@ -766,6 +793,15 @@ void looper_update_cursorposition (looper_gui_t* looper){
         	}
 
 	}
+
+       	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(looper->playbutton)))
+                if (!looperdata_get_playing(looper->data))
+                        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(looper->playbutton),FALSE);
+
+        if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(looper->recbutton)))
+                if (!looperdata_get_recording(looper->data))
+                        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(looper->recbutton),FALSE);
+
 	gtk_loopview_update_cursorposition(GTK_LOOPVIEW(looper->loopview));	
 	return;
 }
@@ -781,6 +817,43 @@ static void looper_gridsettings_callback (GtkMenuItem *menuitem, gpointer data){
         griddialog_new(looper->parent, GTK_LOOPVIEW(looper->loopview));
 }
 
+static void looper_gridfromplayselection_callback (GtkMenuItem *menuitem, gpointer data){
+	looper_gui_t *looper = (looper_gui_t *)data;
+	long s, e, o;
+	if (looper->data){
+		if (looper->data->buf){
+		 	s = looperdata_get_loopstart(looper->data);
+			e = looperdata_get_loopend(looper->data);
+			o = s % (e - s);
+/*			printf ("o:%ld, s:%ld, e:%ld, l:%ld\n",(long)o,(long)s,(long)e,(long)(e - s));*/
+			gtk_loopview_set_bpm(GTK_LOOPVIEW(looper->loopview), (gdouble)(looper->samplerate  * 60.)/ (gdouble)(e - s));
+			gtk_loopview_set_bpb(GTK_LOOPVIEW(looper->loopview),1);
+			gtk_loopview_set_gridoffset(GTK_LOOPVIEW(looper->loopview),o);
+/*			 printf ("o:%ld, bpm:%ld, bpb:%ld\n",
+				(long)gtk_loopview_get_gridoffset(GTK_LOOPVIEW(looper->loopview)),
+				(long)gtk_loopview_get_bpm(GTK_LOOPVIEW(looper->loopview)),
+				(long)gtk_loopview_get_bpb(GTK_LOOPVIEW(looper->loopview)));*/
+		}
+	}
+}
+
+static void looper_gridfromrecselection_callback (GtkMenuItem *menuitem, gpointer data){
+        looper_gui_t *looper = (looper_gui_t *)data;
+        long s, e, o;
+        if (looper->data){
+                if (looper->data->buf){
+                        s = looperdata_get_recstart(looper->data);
+                        e = looperdata_get_recend(looper->data);
+                        o = s % (e - s);
+                        gtk_loopview_set_bpm(GTK_LOOPVIEW(looper->loopview), (gdouble)(looper->samplerate  * 60.)/ (gdouble)(e - s));
+                        gtk_loopview_set_bpb(GTK_LOOPVIEW(looper->loopview),1);
+                        gtk_loopview_set_gridoffset(GTK_LOOPVIEW(looper->loopview),o);
+                }
+        }
+}
+
+
+
 static void looper_custommode_destroy_callback(GtkWidget *dialog, gpointer data){
 /*	looper_gui_t *looper = (looper_gui_t *)data; 
 	this doesn't work because data=responsecode*/
@@ -1097,6 +1170,17 @@ static void looper_playmode_callback(GtkWidget *menu_item, gpointer data ){
 	}
 }
 
+static void looper_recmode_callback(GtkWidget *menu_item, gpointer data ){
+        looper_gui_t *looper = (looper_gui_t*)data;
+
+        if (GTK_CHECK_MENU_ITEM(menu_item)->active) {
+                looperdata_lock(looper->data);
+                looperdata_set_recmode(looper->data,
+                        GPOINTER_TO_UINT(g_object_get_data (G_OBJECT(menu_item),"recmode")));
+                looperdata_unlock(looper->data);
+        }
+}
+
 static void looper_volslider_callback (GtkWidget *volslider, gpointer data ){
 	looper_gui_t *looper = (looper_gui_t*)data;
 	float f/*,f2,f3*/;
@@ -1129,15 +1213,15 @@ static void looper_speedspin_callback (GtkWidget *speedspin, gpointer data ){
 			
 			
 			looper->data->loopend = looper->data->loopstart + (long)ltmp;
-			if (looper->data->loopend > looper->data->buf->size){
-                                looper->data->loopend = looper->data->buf->size;	
+			if (looper->data->loopend > buffer_get_size(looper->data->buf)){
+                                looper->data->loopend = buffer_get_size(looper->data->buf);	
                         }
 			if (looper->data->linkrec2play){
 				looper->data->recend = looper->data->loopend;
 			}else{
 				looper->data->recend = looper->data->recstart + (long)rtmp;
-				if (looper->data->recend > looper->data->buf->size){
-                                        looper->data->recend = looper->data->buf->size;
+				if (looper->data->recend > buffer_get_size(looper->data->buf)){
+                                        looper->data->recend = buffer_get_size(looper->data->buf);
                                 }
 			}
 		}
@@ -1151,21 +1235,21 @@ static void looper_speedspin_callback (GtkWidget *speedspin, gpointer data ){
 			looper->loopendblock = 1;
 			looper->looplengthblock = 1;
 			gtk_spin_button_set_range((GtkSpinButton*)looper->loopstartspin,
-                		0,samples2speedsecs(looper,looper->data->buf->size));
+                		0,samples2speedsecs(looper,buffer_get_size(looper->data->buf)));
         		gtk_spin_button_set_range((GtkSpinButton*)looper->loopendspin,
-                		0,samples2speedsecs(looper,looper->data->buf->size));
+                		0,samples2speedsecs(looper,buffer_get_size(looper->data->buf)));
         		gtk_spin_button_set_range((GtkSpinButton*)looper->looplengthspin,
-                		0,samples2speedsecs(looper,looper->data->buf->size));
+                		0,samples2speedsecs(looper,buffer_get_size(looper->data->buf)));
 
 			looper->recstartblock = 1;
                         looper->recendblock = 1;
                         looper->reclengthblock = 1;
 			gtk_spin_button_set_range((GtkSpinButton*)looper->recstartspin,
-				0,samples2secs(looper,looper->data->buf->size));
+				0,samples2secs(looper,buffer_get_size(looper->data->buf)));
 			gtk_spin_button_set_range((GtkSpinButton*)looper->recendspin,
-                               	0,samples2secs(looper,looper->data->buf->size));
+                               	0,samples2secs(looper,buffer_get_size(looper->data->buf)));
 			gtk_spin_button_set_range((GtkSpinButton*)looper->reclengthspin,
-                               	0,samples2secs(looper,looper->data->buf->size));
+                               	0,samples2secs(looper,buffer_get_size(looper->data->buf)));
 			
 
 			looper->looplengthblock = 1;	
@@ -1300,9 +1384,9 @@ static void looper_looplengthspin_callback (GtkWidget *looplengthspin, gpointer
                 looper->looplengthblock = 0;
         }else{
 		if (looper->data->buf){
-			if ((samples + looper->data->loopstart) > looper->data->buf->size){
+			if ((samples + looper->data->loopstart) > buffer_get_size(looper->data->buf)){
 				looperdata_lock(looper->data);
-				looper->data->loopend = looper->data->buf->size;
+				looper->data->loopend = buffer_get_size(looper->data->buf);
 				looperdata_unlock(looper->data);
 			}else{
 				looperdata_lock(looper->data);
@@ -1406,8 +1490,8 @@ static void looper_reclengthspin_callback (GtkWidget *reclengthspin, gpointer da
         }else{
 		if (looper->data->buf){
 			looperdata_lock(looper->data);
-                	if ((samples + looper->data->recstart) > looper->data->buf->size){
-                        	looper->data->recend = looper->data->buf->size;
+                	if ((samples + looper->data->recstart) > buffer_get_size(looper->data->buf)){
+                        	looper->data->recend = buffer_get_size(looper->data->buf);
 			}else{
 				looper->data->recend = looper->data->recstart + samples;
 			}
@@ -1433,6 +1517,7 @@ static void looper_reclengthspin_callback (GtkWidget *reclengthspin, gpointer da
 looper_gui_t* looper_new(GtkWidget* parent, speciallist_t *looperdatalist, int id){
         looper_gui_t  *looper;
 	GSList 	      *group = NULL;
+	GSList	      *recgroup = NULL;
         GtkAdjustment *adj;
 	GtkWidget     *menu_items;
 	GtkWidget     *menu1;
@@ -1458,6 +1543,15 @@ looper_gui_t* looper_new(GtkWidget* parent, speciallist_t *looperdatalist, int i
 	looper->speedblock = 0;
 	looper->vublock = 0;
 
+#if 0
+	/* try: */
+        /* LATER: replayce with gui */
+	printf ("set\n");	
+        looperdata_set_reverblength(looper->data,44100);
+        looperdata_set_feedback(looper->data,.8);
+	printf ("done\n");
+#endif
+
 	
 	/* widget initialisation */
 	looper->customcurvedialog = NULL;
@@ -1752,6 +1846,20 @@ looper_gui_t* looper_new(GtkWidget* parent, speciallist_t *looperdatalist, int i
         gtk_menu_shell_append (GTK_MENU_SHELL (menu2), looper->showgriditem);
         gtk_widget_show (looper->showgriditem);
 
+	menu_items =  gtk_menu_item_new_with_label ("grid from play selection");
+	g_signal_connect (G_OBJECT (menu_items), "activate",
+						G_CALLBACK (looper_gridfromplayselection_callback),
+                                                (gpointer)looper);
+        gtk_menu_shell_append (GTK_MENU_SHELL (menu2), menu_items);
+        gtk_widget_show (menu_items);
+	
+	menu_items =  gtk_menu_item_new_with_label ("grid from rec selection");
+        g_signal_connect (G_OBJECT (menu_items), "activate",
+                                                G_CALLBACK (looper_gridfromrecselection_callback),
+                                                (gpointer)looper);
+        gtk_menu_shell_append (GTK_MENU_SHELL (menu2), menu_items);
+        gtk_widget_show (menu_items);
+
         menu_items =  gtk_menu_item_new_with_label ("grid settings");
         g_signal_connect (G_OBJECT (menu_items), "activate",
                                                 G_CALLBACK (looper_gridsettings_callback),
@@ -1813,6 +1921,16 @@ looper_gui_t* looper_new(GtkWidget* parent, speciallist_t *looperdatalist, int i
         gtk_menu_shell_append (GTK_MENU_SHELL (menu3), looper->backforthmode);
         gtk_widget_show (looper->backforthmode);
 
+        looper->oncemode =  gtk_radio_menu_item_new_with_label (group,"once");
+        group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (looper->oncemode));
+        gtk_check_menu_item_set_active ((GtkCheckMenuItem*)looper->oncemode,FALSE);
+        g_object_set_data (G_OBJECT (looper->oncemode), "playmode",GUINT_TO_POINTER(LOOP_PLAYMODE_ONCE));
+        g_signal_connect (G_OBJECT (looper->oncemode), "activate",
+                                                G_CALLBACK (looper_playmode_callback),
+                                                (gpointer)looper);
+        gtk_menu_shell_append (GTK_MENU_SHELL (menu3), looper->oncemode);
+        gtk_widget_show (looper->oncemode);
+
 	looper->sinewavemode =  gtk_radio_menu_item_new_with_label (group,"sinewave");
 	group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (looper->sinewavemode));
         gtk_check_menu_item_set_active ((GtkCheckMenuItem*)looper->sinewavemode,FALSE);
@@ -1864,9 +1982,33 @@ looper_gui_t* looper_new(GtkWidget* parent, speciallist_t *looperdatalist, int i
         gtk_menu_shell_append (GTK_MENU_SHELL (menu3), menu_items);
         gtk_widget_show (menu_items);
 
+        menu_items = gtk_separator_menu_item_new();
+        gtk_menu_shell_append (GTK_MENU_SHELL (menu3), menu_items);
+        gtk_widget_show (menu_items);
+
+        looper->recloopmode = gtk_radio_menu_item_new_with_label (recgroup,"looped recording");
+        recgroup = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (looper->recloopmode));
+        g_object_set_data (G_OBJECT (looper->recloopmode), "recmode",GUINT_TO_POINTER(LOOP_RECMODE_LOOP));
+        gtk_check_menu_item_set_active ((GtkCheckMenuItem*)looper->recloopmode,TRUE);
+        g_signal_connect (G_OBJECT (looper->recloopmode), "activate",
+                                                G_CALLBACK (looper_recmode_callback),
+                                                looper);
+        gtk_menu_shell_append (GTK_MENU_SHELL (menu3), looper->recloopmode);
+        gtk_widget_show (looper->recloopmode);
+
+        looper->reconcemode =  gtk_radio_menu_item_new_with_label (recgroup,"single recording");
+        recgroup = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (looper->reconcemode));
+        gtk_check_menu_item_set_active ((GtkCheckMenuItem*)looper->reconcemode,FALSE);
+        g_object_set_data (G_OBJECT (looper->reconcemode), "recmode",GUINT_TO_POINTER(LOOP_RECMODE_ONCE));
+        g_signal_connect (G_OBJECT (looper->reconcemode), "activate",
+                                                G_CALLBACK (looper_recmode_callback),
+                                                (gpointer)looper);
+        gtk_menu_shell_append (GTK_MENU_SHELL (menu3), looper->reconcemode);
+        gtk_widget_show (looper->reconcemode);
+
 	gtk_widget_show (menu3);
 
-        looper->playmodemenu = gtk_menu_item_new_with_label ("playmode");
+        looper->playmodemenu = gtk_menu_item_new_with_label ("modes");
         gtk_menu_item_set_submenu (GTK_MENU_ITEM (looper->playmodemenu), menu3);
         gtk_widget_show (looper->playmodemenu);
 
